﻿using Microsoft.Extensions.Caching.Distributed;
using SMP.Data.Compute;
using SMP.Model;
using SMP.Model.Compute;
using SMP.Model.Enums;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XYY.Common.Standard;

namespace SMP.Service.Compute
{
    public interface IComputeExpenseItemService
    {

        Task<PageResult<View_ComputeExpreseItemDetail>> QueryComputeExpenseItem(QueryModel queryModel);
        Task SaveItemDetail(View_ComputeExpreseItemDetail ComputeExpense);
        Task DeleteComputeExpenseItemDetail(View_ComputeExpreseItemDetail ComputeExpense);
        Task DeleteComputeExpenseItem(View_ComputeExpreseItemDetail ComputeExpense);
        Task<List<Compute_ExpenseItem>> GetAll(int? ServiceProviderId);

        Task<Compute_ExpenseItem> GetExpenseItem(int itemId);
        Task<IEnumerable<Compute_ExpenseItem>> GetExpenseItemByZoneMethodMode(ZoneMode Mode, List<int> ExpenseItemIds);
    }

    public class ComputeExpenseItemService : IComputeExpenseItemService
    {
        private readonly IDistributedCache _cache;
        private readonly ICompute_ExpenseItemRepository expenseRepository;
        private readonly ICompute_ExpenseItemDetailRepository detailRepository;
        private readonly IComputeExpenseService expenseService;
        private readonly IServiceProviderService _serviceProviderService;
        public ComputeExpenseItemService(ICompute_ExpenseItemRepository expenseRepository, ICompute_ExpenseItemDetailRepository detailRepository, IComputeExpenseService expenseService, IDistributedCache cache, IServiceProviderService serviceProviderService)
        {
            this.expenseRepository = expenseRepository;
            this.detailRepository = detailRepository;
            this.expenseService = expenseService;
            this._cache = cache;
            _serviceProviderService = serviceProviderService;
        }

        public async Task DeleteComputeExpenseItemDetail(View_ComputeExpreseItemDetail expenseItem)
        {
            if (expenseItem == null)
                throw new ArgumentNullException($"{expenseItem}不可为空");
            var count = await this.detailRepository.CountAsync(x => x.ItemId == expenseItem.Id);
            if (count == 1)
                await this.expenseRepository.DeleteByIdAsync(expenseItem.Id);
            await this.detailRepository.DeleteByIdAsync(expenseItem.DetailId);
            //缓存更新
            RefreshCache(expenseItem.Id);
        }

        public async Task<List<Compute_ExpenseItem>> GetAll(int? ServiceProviderId)
        {
            var list = new List<Compute_ExpenseItem>();
            if (ServiceProviderId == null)
                list = (await this.expenseRepository.QueryAsync()).ToList();
            else
                list = (await this.expenseRepository.QueryAsync(x => x.ServiceProviderId == ServiceProviderId)).ToList();
            var splist = await _serviceProviderService.GetProviderList();
            list.ForEach(x =>
            {
                var sp = splist.Where(y => y.Id == x.ServiceProviderId).FirstOrDefault();
                if (sp != null)
                {
                    x.FullName = sp.ServiceName + "-" + x.Name;
                }
                else
                {
                    x.FullName = x.Name;
                }

            });
            return list;
        }


        public async Task SaveItem(View_ComputeExpreseItemDetail expenseItem)
        {

            if (expenseItem == null)
                throw new ArgumentNullException($"{expenseItem}不可为空");
            StringBuilder errorLine = new StringBuilder();

            if (string.IsNullOrEmpty(expenseItem.Name))
            {
                errorLine.AppendLine("费项名称必填");
            }
            if (expenseItem.ServiceProviderId == 0)
            {
                errorLine.AppendLine("服务商名称必填");
            }
            if (expenseItem.CategoryId == 0)
            {
                errorLine.AppendLine("费项必填");
            }

            if (await expenseRepository.IsExistsAsync(x => x.Name == expenseItem.Name && x.Id != expenseItem.Id))
            {
                errorLine.AppendLine($"添加费项{expenseItem.Name}失败，已存在系统内，请勿重复添加。");
            }

            RefreshCache(expenseItem.Id);
            await expenseRepository.UpdateAsync(expenseItem);
        }



        public async Task SaveItemDetail(View_ComputeExpreseItemDetail expenseItem)
        {
            if (expenseItem == null)
                throw new ArgumentNullException($"{expenseItem}不可为空");
            StringBuilder errorLine = new StringBuilder();


            //基础Id
            if (string.IsNullOrEmpty(expenseItem.Name))
            {
                errorLine.AppendLine("费项名称必填");
            }
            if (expenseItem.ServiceProviderId == 0)
            {
                errorLine.AppendLine("服务商名称必填");
            }
            if (expenseItem.CategoryId == 0)
            {
                errorLine.AppendLine("费项必填");
            }
            if (expenseItem.Id == 0)
            {
                if (await expenseRepository.IsExistsAsync(x => x.Name == expenseItem.Name))
                {
                    //expenseItem.Id = (await expenseRepository.QueryAsync(x => x.Name == expenseItem.Name)).First().Id;
                    errorLine.AppendLine($"添加费项{expenseItem.Name}失败，已存在系统内，请勿重复添加。");
                }
                else
                {
                    //添加费项
                    expenseItem.Id = (int)await expenseRepository.InsertAsync(expenseItem);
                }
            }
            else
            {
                //缓存更新
                RefreshCache(expenseItem.Id);
                //更新费项
                await expenseRepository.UpdateAsync(expenseItem);
            }


            if (expenseItem.ExpenseId == 0)
            {
                errorLine.AppendLine("没有选择费用明细");
            }
            if (expenseItem.ExpenseId == 0)
            {
                errorLine.AppendLine("未选择计费顺序");
            }

            if (errorLine.Length != 0)
                throw new Exception(errorLine.ToString());

            var ed = new Compute_ExpenseItemDetail
            {
                ExpenseId = expenseItem.ExpenseId,
                ItemId = expenseItem.Id,
                Sort = expenseItem.Sort,
                Id = expenseItem.DetailId
            };
            if (expenseItem.DetailId == 0)
            {
                await detailRepository.InsertAsync(ed);
            }
            else
            {
                await detailRepository.UpdateAsync(ed);
            }
        }

        public async Task<PageResult<View_ComputeExpreseItemDetail>> QueryComputeExpenseItem(QueryModel expense)
        {
            return await this.detailRepository.QueryComputeExpreseItemDetail(expense);
        }

        public string @computeItemKey => "expense#";

        public void RefreshCache(int itemId)
        {
            this._cache.Remove(computeItemKey + itemId.ToString());
        }

        public async Task DeleteComputeExpenseItem(View_ComputeExpreseItemDetail detail)
        {
            if (detail == null)
                throw new ArgumentNullException($"{detail}不可为空");

            //缓存更新
            RefreshCache(detail.Id);
            await expenseRepository.DeleteByIdAsync(detail.Id);
            await detailRepository.DeleteAsync(x => x.ItemId == detail.Id);
        }

        public async Task<Compute_ExpenseItem> GetExpenseItem(int itemId)
        {
            var item = await expenseRepository.GetAsync(itemId);
            var details = await detailRepository.QueryAsync(x => x.ItemId == itemId);
            item.ItemDetails = details.OrderBy(x => x.Sort).ToList();
            int[] detailIds = item.ItemDetails.Select(x => x.ExpenseId).ToArray();
            var dicst = await expenseService.GetExpenseVersion(detailIds);
            item.ItemDetails.ForEach(x =>
            {
                if (dicst.ContainsKey(x.ExpenseId))
                {
                    x.DetailVersions = dicst[x.ExpenseId];
                }
            });
            return item;
        }
        public async Task<IEnumerable<Compute_ExpenseItem>> GetExpenseItemByZoneMethodMode(ZoneMode Mode, List<int> ExpenseItemIds)
        {
            return await expenseRepository.GetExpenseItemByZoneMethodMode(Mode, ExpenseItemIds);
        }
    }
}
