﻿using Microsoft.Extensions.Caching.Distributed;
using SMP.Common.Unit;
using SMP.Data.Compute;
using SMP.Model.Compute;
using SMP.Model.Dto.Finance;
using SMP.Model.Enums;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XYY.Common.Standard;
using XYY.Core.Standard.Data.Infrastructure;
using Dapper;

namespace SMP.Service.Compute
{
    public interface IComputeExpenseService
    {

        Task<PageResult<Compute_Expense>> QueryComputeExpense(QueryModel queryModel);
        Task<int> AddComputeVersionExpense(Compute_Expense ComputeExpense);
        Task UpdateComputeVersionExpense(Compute_Expense ComputeExpense);
        Task DeleteComputeExpenseVersion(int VersionId);

        Task InsertComputeExpense(Compute_Expense ComputeExpense);
        Task DeleteComputeExpense(Compute_Expense ComputeExpense);
        Task<List<Compute_Expense>> GetAll();
        Task<IEnumerable<Compute_Expense>> GetByProviderId(int providerId);
        Task<Compute_Expense> GetBase(int id);
        Task UpdateComputeExpense(Compute_Expense expense);

        //Task<Compute_Expense> Get(int Id);

        Task<Compute_Expense> GetVersion(int VersionId);
        Task<Dictionary<int, List<Compute_ExpenseVersion>>> GetVersion(int[] expenseId);
        Task<Dictionary<int, List<Compute_Expense>>> GetExpenseVersion(int[] detailIds);
        Task RefreshItemCache(params int[] expenseIds);
        Task<IEnumerable<QueryExpenseByLadingResult>> GetExpenseWithVersionByLading(QueryExpenseByLadingInput input);
        Task<IEnumerable<Compute_ExpenseItem>> GetAllExpenseItemByTemplateId(int TemplateId);
    }

    public class ComputeExpenseService : IComputeExpenseService
    {
        private readonly IUnitOfWork _unitOfWork;
        private readonly ICompute_ExpenseRepository expenseRepository;
        private readonly ICompute_ExpenseDetailZoneRepository detailZoneRepository;
        private readonly ICompute_ExpenseDetailRepository detailRepository;
        private readonly ICompute_ExpenseZoneRepository expenseZoneRepository;
        private readonly ICompute_ZoneRepository zoneRepository;
        private readonly ICompute_ExpenseVersionRepository versionRepository;
        private readonly ICompute_ZoneMethodRepository methodRepository;
        private readonly ICompute_FormulaRepository formulaRepository;
        private readonly ICompute_ExpenseItemDetailRepository itemDetailRepository;
        private readonly IDistributedCache _cache;
        public ComputeExpenseService(ICompute_ExpenseRepository expenseRepository, ICompute_ExpenseDetailZoneRepository detailZoneRepository, ICompute_ExpenseDetailRepository detailRepository, ICompute_ExpenseZoneRepository expenseZoneRepository, ICompute_ZoneRepository zoneRepository, ICompute_ExpenseVersionRepository versionRepository, ICompute_ZoneMethodRepository methodRepository, ICompute_FormulaRepository formulaRepository, IDistributedCache cache, ICompute_ExpenseItemDetailRepository itemDetailRepository
            , IUnitOfWork unitOfWork)
        {
            this.expenseRepository = expenseRepository;
            this.detailZoneRepository = detailZoneRepository;
            this.detailRepository = detailRepository;
            this.expenseZoneRepository = expenseZoneRepository;
            this.zoneRepository = zoneRepository;
            this.versionRepository = versionRepository;
            this.methodRepository = methodRepository;
            this.formulaRepository = formulaRepository;
            _cache = cache;
            this.itemDetailRepository = itemDetailRepository;
            this._unitOfWork = unitOfWork;
        }

        public async Task DeleteComputeExpense(Compute_Expense expense)
        {
            await RefreshItemCache(expense.Id);
            await this.expenseRepository.UpdateByConditionAsync(x => x.Id == expense.Id, new { IsDelete = true });
            await this.detailRepository.UpdateByConditionAsync(x => x.ExpenseID == expense.Id, new { IsDelete = true });
            await this.detailZoneRepository.UpdateByConditionAsync(x => x.ExpenseId == expense.Id, new { IsDelete = true });
            await this.expenseZoneRepository.UpdateByConditionAsync(x => x.ExpenseId == expense.Id, new { IsDelete = true });
            await this.versionRepository.UpdateByConditionAsync(x => x.ExpenseId == expense.Id, new { IsDelete = true });
        }

        public async Task DeleteComputeExpenseVersion(int VersionId)
        {
            var verison = await this.versionRepository.GetAsync(VersionId);
            if (verison == null)
                throw new Exception($"未找到版本[VersionId]#{VersionId}");
            var expense = await this.expenseRepository.GetAsync(verison.ExpenseId);
            await this.detailRepository.UpdateByConditionAsync(x => x.VersionId == VersionId, new { IsDelete = true });
            await this.detailZoneRepository.UpdateByConditionAsync(x => x.VersionId == VersionId, new { IsDelete = true });
            await this.expenseZoneRepository.UpdateByConditionAsync(x => x.VersionId == VersionId, new { IsDelete = true });
            await this.versionRepository.UpdateByConditionAsync(x => x.Id == VersionId, new { IsDelete = true });

            //更新最后版本ID与版本时间
            var d = (await this.versionRepository
                .QueryBySqlAsync("select top 1 * from  Compute_ExpenseVersion(nolock) where IsDelete=0 and ExpenseId=@ExpenseId order by VersionDate desc",
                new { ExpenseId = verison.ExpenseId }))
                .FirstOrDefault();

            if (d != null && expense != null)
            {
                expense.LastVersionDate = d.VersionDate;
                expense.LastVersionId = d.Id;
                await this.expenseRepository.UpdateAsync(expense);
                await RefreshItemCache(expense.Id);
            }
            else
            {
                throw new Exception("每个费用至少存在一个计费版本，您可以删除整个费用!");
            }
        }

        public async Task<Compute_Expense> GetVersion(int VersionId)
        {
            if (VersionId == 0)
                throw new Exception("必须传入VersionId");
            var version = await versionRepository.GetAsync(VersionId);
            if (version == null)
                throw new Exception($"未找到相关的版本Id:{VersionId}");
            if (version.IsDelete)
                throw new Exception($"VersionId#{VersionId}已被删除");
            var exprense = await this.expenseRepository.GetAsync(version.ExpenseId);
            if (exprense == null)
                throw new Exception($"未找到相关的费用Id:{version.ExpenseId}");

            exprense.BindBillId = version.BindBillId;
            exprense.VersionId = version.Id;
            exprense.VersionDate = version.VersionDate;
            exprense.Details = (await detailRepository.QueryAsync(x => x.VersionId == VersionId && x.IsDelete == false)).OrderBy(x => x.Weight).ToList();
            int[] fromid = exprense.Details.Select(x => x.FormulaId).ToArray();
            var formulas = (await formulaRepository.QueryAsync(x => x.Id.In(fromid))).ToList();
            foreach (var x in exprense.Details)
            {
                x.ToG = x.WeightUnit switch
                {
                    WeightUnit.OZ => (int)Math.Round(x.Weight * ConvertWeightHelper.OZToG, MidpointRounding.AwayFromZero),
                    WeightUnit.LBS => (int)Math.Round(x.Weight * ConvertWeightHelper.LBSToG, MidpointRounding.AwayFromZero),
                    WeightUnit.KG => (int)Math.Round(x.Weight * ConvertWeightHelper.KGToG, MidpointRounding.AwayFromZero),
                    WeightUnit.G => (int)Math.Round(x.Weight, MidpointRounding.AwayFromZero),
                    _ => (int)Math.Round(x.Weight, MidpointRounding.AwayFromZero)
                };
                if (x.WeightUnit == WeightUnit.OZ && x.Weight == 16)
                {
                    x.ToG = 453;
                }

                x.ZoneValue = (await detailZoneRepository.QueryAsync(y => y.ExpenseDetailId == x.Id && y.IsDelete == false)).ToList();
                x.Formula = formulas.Where(y => y.Id == x.FormulaId).FirstOrDefault();
            }
            exprense.Details = exprense.Details.OrderBy(x => x.ToG).ToList();
            int[] zoneIds = (await expenseZoneRepository.QueryAsync(x => x.VersionId == VersionId && x.IsDelete == false)).Select(x => x.ZoneId).ToArray();
            if (zoneIds.Length > 0)
            {
                exprense.Zones = (await zoneRepository.QueryAsync(x => x.Id.In(zoneIds))).ToList();
                var methods = (await methodRepository.QueryAsync(x => x.ZoneId.In(zoneIds))).ToList();
                exprense.Zones.ForEach(y =>
                {
                    y.ZoneMethods = methods.Where(x => x.ZoneId == y.Id).ToList();
                });
            }

            return exprense;
        }

        public async Task<int> AddComputeVersionExpense(Compute_Expense expense)
        {
            if (expense == null)
                throw new Exception($"必须传入expense");
            if (expense.Id == 0)
                throw new Exception($"必须传入expense.Id");
            if (expense.VersionDate == null)
                throw new Exception("版本日期[VersionDate]不可为空");
            string existsSql = "select * from Compute_ExpenseVersion(nolock) where ExpenseId=@ExpenseId and VersionDate=@VersionDate and BindBillId=@BindBillId";
            if (expense.BindBillId == 0)
            {
                //如果绑定过提单Id的，我们不做日期版本限制
                int countQty = await versionRepository.ExecuteScalarAsync<int>(existsSql, new
                {
                    BindBillId = expense.BindBillId,
                    ExpenseId = expense.Id,
                    VersionDate = expense.VersionDate.Value.ToString("yyyy-MM-dd")
                });
                if (countQty > 0)
                    throw new Exception($"不可以存在同一个版本{expense.VersionDate}");
            }
            else
            {
                //绑定提单Id的，我们以最后一次为准，直接做更新操作
                var last = (await versionRepository.QueryAsync(x => x.ExpenseId == expense.Id && x.BindBillId == expense.BindBillId));
                if (last.Count() > 0)
                {
                    expense.VersionId = last.First().Id;
                    await UpdateComputeVersionExpense(expense);
                    return expense.VersionId.Value;
                }
            }
            if (expense.Zones == null)
                throw new Exception("分区[Zones]不可为空");

            if (expense.Details == null)
                throw new Exception("计费明细[Details]不可为空");
            if (expense.Min < expense.Stepping)
                throw new Exception("最小值不应小于步进值");
            if (expense.CurrenyId == 0)
                throw new Exception("必须传入币种");

            int VersionId = (int)await this.versionRepository.InsertAsync(
                new Compute_ExpenseVersion
                {
                    VersionDate = expense.VersionDate,
                    ExpenseId = expense.Id,
                    BindBillId = expense.BindBillId
                });


            await this.expenseRepository.UpdateAsync(expense);
            foreach (var x in expense.Details)
            {

                if (expense.ComputeType == ComputeType.按固定价)
                {
                    //按固定价时，我们直接设置明细重量上限为99999KG
                    x.Weight = 99999;
                    x.WeightUnit = WeightUnit.KG;
                }

                x.ExpenseID = expense.Id;
                x.VersionId = VersionId;
                int detailId = (int)await this.detailRepository.InsertAsync(x);
                x.ZoneValue = x.ZoneValue.Where(x => x.Fee != 0).ToList();
                x.ZoneValue.ForEach(x =>
                {
                    x.ExpenseDetailId = detailId;
                    x.ExpenseId = expense.Id;
                    x.CreateTime = DateTime.Now;
                    x.VersionId = VersionId;
                });
                await this.detailZoneRepository.BulkToDBAsync(x.ZoneValue);
                var appendZones = (expense.Zones.Select(x =>
                new Compute_ExpenseZone
                {
                    ExpenseId = expense.Id,
                    CreateTime = DateTime.Now,
                    ZoneId = x.Id,
                    VersionId = VersionId
                })).ToList();
                await this.expenseZoneRepository.BulkToDBAsync(appendZones);
            }

            var d = (await this.versionRepository
                .QueryBySqlAsync("select top 1 * from  Compute_ExpenseVersion(nolock) where ExpenseId=@ExpenseId  and IsDelete=0 And BindBillId =0 order by VersionDate desc",
                new { ExpenseId = expense.Id }))
                .FirstOrDefault();
            if (d != null)
            {
                expense.LastVersionDate = d.VersionDate;
                expense.LastVersionId = d.Id;
                await this.expenseRepository.UpdateAsync(expense);
            }
            await RefreshItemCache(expense.Id);
            return VersionId;
        }

        public async Task<List<Compute_Expense>> GetAll()
        {
            return (await this.expenseRepository.QueryAsync(x => x.IsDelete == false)).ToList();
        }

        public async Task InsertComputeExpense(Compute_Expense expense)
        {



            if (expense == null)
                throw new Exception($"必须传入expense");

            if (await this.expenseRepository.IsExistsAsync(x => x.Name == expense.Name))
                throw new Exception($"费用名称[Name]不可重复");

            if (expense.VersionDate == null)
                throw new Exception("版本日期[VersionDate]不可为空");

            if (expense.Zones == null)
                throw new Exception("分区[Zones]不可为空");

            if (expense.Details == null)
                throw new Exception("计费明细[Details]不可为空");

            expense.LastVersionDate = expense.VersionDate;
            expense.LastVersionId = 0;

            int ExpenseId = (int)await this.expenseRepository.InsertAsync(expense);
            int VersionId = (int)await this.versionRepository.InsertAsync(
                new Compute_ExpenseVersion
                {
                    ExpenseId = ExpenseId,
                    VersionDate = expense.VersionDate
                });
            expense.LastVersionId = VersionId;
            expense.Id = ExpenseId;
            await this.expenseRepository.UpdateAsync(expense);

            foreach (var x in expense.Details)
            {
                if (expense.ComputeType == ComputeType.按固定价)
                {
                    //按固定价时，我们直接设置明细重量上限为99999KG
                    x.Weight = 99999;
                    x.WeightUnit = WeightUnit.KG;
                }

                if (x.WeightUnit == Model.Enums.WeightUnit.KG && x.Weight > 10000 * 1000)
                {
                    throw new Exception("重量不可大于10000000KG");
                }
                if (x.WeightUnit == Model.Enums.WeightUnit.G && x.Weight > 1000 * 1000 * 1000)
                {
                    throw new Exception("重量不可大于1000000000G，可尝试使用KG单位");
                }
                if (x.WeightUnit == Model.Enums.WeightUnit.OZ && x.Weight > 10000 * 1000)
                {
                    throw new Exception("重量不可大于10,000,000OZ");
                }
                if (x.WeightUnit == Model.Enums.WeightUnit.LBS && x.Weight > 10000 * 200)
                {
                    throw new Exception("重量不可大于200,000LBS");

                }

                x.ZoneValue = x.ZoneValue.Where(y => expense.Zones.Any(z => z.Id == y.ZoneId)).ToList();
                x.ExpenseID = ExpenseId;
                x.VersionId = VersionId;
                int detailId = (int)await this.detailRepository.InsertAsync(x);
                x.ZoneValue.ForEach(y =>
                {
                    y.ExpenseDetailId = detailId;
                    y.ExpenseId = ExpenseId;
                    y.CreateTime = DateTime.Now;
                    y.VersionId = x.VersionId;
                });

                await this.detailZoneRepository.BulkToDBAsync(x.ZoneValue);

            }
            var appendZones = (expense.Zones.Select(y => new Compute_ExpenseZone
            {
                ExpenseId = ExpenseId,
                CreateTime = DateTime.Now,
                ZoneId = y.Id,
                VersionId = VersionId
            })).ToList();
            await this.expenseZoneRepository.BulkToDBAsync(appendZones);
        }

        public async Task<PageResult<Compute_Expense>> QueryComputeExpense(QueryModel expense)
        {
            if (expense.QueryParamer == null)
                expense.QueryParamer = new List<QueryParamer>();
            expense.QueryParamer.Add(new QueryParamer { Filed = "IsDelete", Method = "Eq", Value = "False" });
            var result = await this.expenseRepository.QueryPagerAsync(expense);
            var verids = result.Result.Select(x => x.Id).ToArray();
            var versions = await GetVersion(verids);
            foreach (var item in result.Result)
            {
                if (versions.ContainsKey(item.Id))
                {
                    item.Versions = versions[item.Id].OrderByDescending(x => x.VersionDate).ToList();
                    item.Versions = item.Versions.Where(x => x.BindBillId == 0).ToList();
                }
            }
            return result;
        }


        public async Task<Dictionary<int, List<Compute_Expense>>> GetExpenseVersion(int[] expenseId)
        {
            if (expenseId == null || expenseId.Length == 0)
                return new Dictionary<int, List<Compute_Expense>>();
            string sql = "select * from Compute_ExpenseVersion(nolock) where ExpenseId in @ExpenseIds and IsDelete=0 ";
            var details = (await this.versionRepository.QueryBySqlAsync(sql, new { ExpenseIds = expenseId }))
               .GroupBy(x => x.ExpenseId).ToDictionary(x => x.Key, x => x.OrderBy(y => y.VersionDate).ToList());
            var result = new Dictionary<int, List<Compute_Expense>>();
            foreach (var x in details)
            {
                List<Compute_Expense> expenseList = new List<Compute_Expense>();
                foreach (var y in x.Value)
                {
                    expenseList.Add(await GetVersion(y.Id));
                }
                result.Add(x.Key, expenseList);
            }
            return result;
        }


        public async Task<Dictionary<int, List<Compute_ExpenseVersion>>> GetVersion(int[] expenseId)
        {
            if (expenseId == null || expenseId.Length == 0)
                return new Dictionary<int, List<Compute_ExpenseVersion>>();
            string sql = "select * from Compute_ExpenseVersion(nolock) where ExpenseId in @ExpenseIds and IsDelete=0 ";
            return (await this.versionRepository.QueryBySqlAsync(sql, new { ExpenseIds = expenseId }))
               .GroupBy(x => x.ExpenseId).ToDictionary(x => x.Key, x => x.ToList());
        }

        public async Task UpdateComputeVersionExpense(Compute_Expense expense)
        {
            if (expense == null)
                throw new Exception($"必须传入expense");
            if (expense.Id == 0)
                throw new Exception($"必须传入expense.Id");

            if (expense.VersionDate == null)
                throw new Exception("版本日期[VersionDate]不可为空");

            if (expense.Zones == null)
                throw new Exception("分区[Zones]不可为空");

            if (expense.Details == null)
                throw new Exception("计费明细[Details]不可为空");

            if (expense.VersionId == null)
                throw new Exception("未传入需要变更的版本");


            string existsSql = "select * from Compute_ExpenseVersion(nolock) where ExpenseId=@ExpenseId and VersionDate=@VersionDate and Id<>@Id And BindBillId = @BindBillId";
            int countQty = await versionRepository.ExecuteScalarAsync<int>(existsSql,
                new
                {
                    ExpenseId = expense.Id,
                    VersionDate = expense.VersionDate.Value.ToString("yyyy-MM-dd"),
                    Id = expense.VersionId,
                    BindBillId = expense.BindBillId
                });
            if (countQty > 0)
                throw new Exception($"不可以存在同一个版本{expense.VersionDate}");


            await this.detailRepository.UpdateByConditionAsync(x => x.VersionId == expense.VersionId, new { IsDelete = true });
            await this.detailZoneRepository.UpdateByConditionAsync(x => x.VersionId == expense.VersionId, new { IsDelete = true });
            await this.expenseZoneRepository.UpdateByConditionAsync(x => x.VersionId == expense.VersionId, new { IsDelete = true });

            await this.expenseRepository.UpdateAsync(expense);
            await this.versionRepository.UpdateByConditionAsync(x => x.Id == expense.VersionId, new { VersionDate = expense.VersionDate });
            foreach (var x in expense.Details)
            {

                if (expense.ComputeType == ComputeType.按固定价)
                {
                    //按固定价时，我们直接设置明细重量上限为99999KG
                    x.Weight = 99999;
                    x.WeightUnit = WeightUnit.KG;
                }

                x.ZoneValue = x.ZoneValue.Where(y => expense.Zones.Any(z => z.Id == y.ZoneId)).ToList();
                x.ExpenseID = expense.Id;
                x.VersionId = expense.VersionId.Value;
                int detailId = (int)await this.detailRepository.InsertAsync(x);
                x.ZoneValue.ForEach(x =>
                {
                    x.ExpenseDetailId = detailId;
                    x.ExpenseId = expense.Id;
                    x.CreateTime = DateTime.Now;
                    x.VersionId = expense.VersionId.Value;
                });
                await this.detailZoneRepository.BulkToDBAsync(x.ZoneValue);

            }

            var appendZones = expense.Zones.Select(x => new Compute_ExpenseZone
            {
                ExpenseId = expense.Id,
                CreateTime = DateTime.Now,
                ZoneId = x.Id,
                VersionId = expense.VersionId.Value
            }).ToList();
            await this.expenseZoneRepository.BulkToDBAsync(appendZones);
            //更新最后版本ID与版本时间

            var d = (await this.versionRepository
                .QueryBySqlAsync("select top 1 * from  Compute_ExpenseVersion(nolock) where ExpenseId=@ExpenseId  and IsDelete=0 And BindBillId =0 order by VersionDate desc",
                new { ExpenseId = expense.Id }))
                .FirstOrDefault();
            if (d != null)
            {
                expense.LastVersionDate = d.VersionDate;
                expense.LastVersionId = d.Id;
                await this.expenseRepository.UpdateAsync(expense);
            }
            await RefreshItemCache(expense.Id);
        }

        public async Task<IEnumerable<Compute_Expense>> GetByProviderId(int providerId)
        {
            var result = await expenseRepository.QueryAsync(e => e.ServiceProviderId == providerId);
            return result;
        }

        public string @computeItemKey => "expense#";
        public async Task RefreshItemCache(params int[] expenseIds)
        {
            string sql = "select ItemId from Compute_ExpenseItemDetail(nolock) where ExpenseId in @expenseIds";
            var itemList = await itemDetailRepository.QueryBySqlAsync(sql, new { expenseIds = expenseIds });
            foreach (var item in itemList)
            {
                _cache.Remove(@computeItemKey + item.ItemId);
            }
        }
        public async Task<IEnumerable<QueryExpenseByLadingResult>> GetExpenseWithVersionByLading(QueryExpenseByLadingInput input)
        {
            StringBuilder sb = new StringBuilder();
            string where = string.Empty; string rightJoin = string.Empty; string ExpenseDetailLeftJoin = string.Empty;
            if (!string.IsNullOrEmpty(input.LadingBillNumber))
            {
                rightJoin = " right join Finance_Incidental(nolock) f on f.ExpenseItemId=a.Id and f.VersionId=e.VersionId and f.ExpenseId=b.ExpenseId ";
                where += $" and f.LadingBillId={input.BindBillId} and f.IsDeleted=0 ";
            }
            if (input.BillTime.HasValue)
            {
                ExpenseDetailLeftJoin = $@" outer apply(select top 1 cc1.ExpenseId,cc1.Id from Compute_ExpenseVersion(nolock) cc1 where cc1.ExpenseId=c.Id and cc1.IsDelete=0 and cc1.VersionDate<='{input.BillTime}' and cc1.BindBillId=(select case when num>0 then {input.BindBillId} else 0 end  from (select COUNT(Id) num from Compute_ExpenseVersion(nolock) where BindBillId={input.BindBillId} and ExpenseId=c.Id and IsDelete=0)cc2  ) order by id desc ) c23
left join Compute_ExpenseDetail(nolock) c2 on c2.ExpenseID=c23.ExpenseId and c2.VersionId=c23.Id  and c2.IsDelete=0  ";
            }
            else
            {
                ExpenseDetailLeftJoin = " left join Compute_ExpenseDetail c2 on c2.ExpenseID=c.Id  and c2.IsDelete=0 ";
            }
            sb.Append(@" select a.Id ExpenseItemId,c.Id ExpenseId,c.Name ExpenseName,c.ApportionMethod,d.Id VersionId,d.BindBillId,d.VersionDate,e.Fee IncidentalFee,c1.Name IncidentalCurrency,c2.Id ExpenseDetailId,c2.Weight,c2.WeightType,c2.WeightUnit
,ROW_NUMBER()over(PARTITION by c.Id order by d.BindBillId desc) num  ");
            sb.Append(@$" from Compute_ExpenseItem(nolock) a
 left join Compute_ExpenseItemDetail(nolock) b on b.ItemId=a.Id 
left join Compute_Expense(nolock) c on c.Id=b.ExpenseId and c.IsDelete=0
left join Sys_Currency(nolock) c1 on c1.Id =c.CurrenyId
{ExpenseDetailLeftJoin}
left join Compute_ExpenseDetailZone(nolock) e on e.ExpenseDetailId=c2.Id  and e.IsDelete=0
left join Compute_ExpenseVersion(nolock) d on d.Id=e.VersionId and d.IsDelete=0
{rightJoin} ");
            sb.Append(" where 1=1 ");
            //where条件
            if (input.ExpenseId > 0) { where += $" and c.Id={input.ExpenseId} "; }
            if (input.ExpenseItemId > 0) { where += $" and a.Id={input.ExpenseItemId} "; }
            if (input.VersionId > 0) { where += $" and d.Id={input.VersionId} "; }
            else
            {
                if (!input.BillTime.HasValue)//没有订单时间条件时添加
                {
                    where += $" and (e.VersionId=c.LastVersionId or e.VersionId=(select id from Compute_ExpenseVersion where BindBillId={(input.BindBillId > 0 ? input.BindBillId : -1)} and ExpenseID = c.Id )) ";
                }

            }
            sb.Append(where);
            return await _unitOfWork.QueryBySqlAsync<QueryExpenseByLadingResult>(sb.ToString());
        }
        public async Task<IEnumerable<Compute_ExpenseItem>> GetAllExpenseItemByTemplateId(int TemplateId)
        {
            string sql = $@" select c.* from Compute_ExpenseItemTemplate(nolock) a 
left join Compute_ExpenseItemTemplateDetail(nolock) b on b.TemplateId=a.Id and b.IsDelete=0
left join Compute_ExpenseItem(nolock) c on c.Id=b.Compute_ExpenseItemId 
where a.Id= {TemplateId}";
            return await expenseRepository.QueryAsync<Compute_ExpenseItem>(sql);

        }
        public async Task UpdateComputeExpense(Compute_Expense expense)
        {
            if (expense.Id == 0)
                throw new Exception("请传入费用Id");
            await RefreshItemCache(expense.Id);

            await expenseRepository.UpdateByConditionAsync(x => x.Id == expense.Id, new
            {
                IsDelete = expense.IsDelete,
                ComputeType = expense.ComputeType,
                CurrenyId = expense.CurrenyId,
                ExpenseCategory = expense.ExpenseCategory,
                Min = expense.Min,
                ServiceProviderId = expense.ServiceProviderId,
                Name = expense.Name,
                ApportionMethod = expense.ApportionMethod,
                Stepping = expense.Stepping
            });
        }

        public async Task<Compute_Expense> GetBase(int id)
        {
            return await this.expenseRepository.GetAsync(id);
        }
    }
}
