﻿using RestSharp;
using SMP.Data;
using SMP.Data.Finance;
using SMP.Model;
using SMP.Model.Enums;
using SMP.Model.Finance;
using SMP.Service.BaseInfo;
using SMP.Service.BaseInfo.Dto;
using SMP.Service.Finance;
using SMP.Service.OrderRangeQuery.Dto;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XYY.Common.Standard;

namespace SMP.Service.OrderRangeQuery
{
    public interface IOrderRangeQueryService
    {
        Task<int> SaveSetting(OrderRangeQuerySettingDto dto);
        Task<List<OrderRangeQuerySettingItem>> SearchSettingByName(string name);

        Task<Finance_OrderQuerySetting> InsertOrderRange(int SettingId);

        Task<int> InsertOrderRange(Finance_OrderQuerySetting setting);
        Task<IEnumerable<OrderRangeQuerySettingItem>> GetAllSettings();
        Task<PageResult<OrderRangeQuerySettingDto>> QueryPage(QueryModel queryModel);
        Task<OrderRangeQuerySettingDto> LoadRangeSetting(int id);
        Task<PageResult<OrderRangeResultDto>> QueryOrderResultPage(QueryModel queryModel);
        Task<Finance_OrderQuerySetting> Execute(int id);
        Task<bool> Remove(int id);
        Task<Finance_OrderQuerySetting> GetSettings(int orderRangeSettingId);
    }

    public class OrderRangeQueryService : IOrderRangeQueryService
    {
        IFinance_OrderQuerySettingRepository _orderQuerySettingRepository;
        IFinance_OrderRangeSearchResultRepository _OrderRangeSearchResultRepository;
        IBaseInfoService _baseInfoService;
        IFinance_Logistics_LadingBillService _LadingBillService;
        public OrderRangeQueryService(IFinance_OrderQuerySettingRepository orderQuerySettingRepository, IFinance_OrderRangeSearchResultRepository orderRangeSearchResultRepository, IBaseInfoService baseInfoService
            , IFinance_Logistics_LadingBillService LadingBillService)
        {
            _orderQuerySettingRepository = orderQuerySettingRepository;
            _OrderRangeSearchResultRepository = orderRangeSearchResultRepository;
            _baseInfoService = baseInfoService;
            _LadingBillService = LadingBillService;
        }
        public async Task<int> SaveSetting(OrderRangeQuerySettingDto dto)
        {
            if (await _orderQuerySettingRepository.IsExistsAsync(x=>x.Name==dto.Name&&x.Id!= dto.Id&&x.Deleted==false)|| await _LadingBillService.IsExistsAsync(dto.Name,dto.Id)) 
            { throw new Exception("名称重复，请重新设置名称！"); }
            if (dto.Id <= 0)
            {
                int id = await AddSetting(dto);
                await Execute(id);
                await _LadingBillService.ScopeBillCreate(id,dto.Name);
                return id;
            }
            else
            {
                await UpdateSetting(dto);
                await Execute(dto.Id);
                await _LadingBillService.ScopeBillEditByScop(dto.Name,dto.Id);
                return dto.Id;
            }

        }

        private async Task<bool> UpdateSetting(OrderRangeQuerySettingDto dto)
        {
            await CheckSave(dto, false);

            var entity = CreateEntity(dto);
            entity.Id = dto.Id;
            return await _orderQuerySettingRepository.UpdateAsync(entity);
        }

        private Finance_OrderQuerySetting CreateEntity(OrderRangeQuerySettingDto dto)
        {
            string publicChannelIds = null;
            if (dto.PublicChannelIds?.Any() == true)
            {
                publicChannelIds = string.Join(",", dto.PublicChannelIds);
            }
            string deliveryChannelIds = null;
            if (dto.DeliveryChannelIds?.Any() == true)
            {
                deliveryChannelIds = string.Join(",", dto.DeliveryChannelIds);
            }
            string customerIds = null;
            if (dto.CustomerIds?.Any() == true)
            {
                customerIds = string.Join(",", dto.CustomerIds);
            }
            string notcustomerIds = null;
            if (dto.NotCustomerIds?.Any() == true)
            {
                notcustomerIds = string.Join(",", dto.NotCustomerIds);
            }

            DateTime? from = null, to = null;
            if (dto.TimeRange?.Any() == true)
            {
                if (dto.TimeRange.Count != 2)
                {
                    throw new Exception("时间范围值必须为2个时间值");
                }
                else
                {
                    if (from > to)
                        throw new Exception("开始时间不能晚于结束时间");

                    from = dto.TimeRange[0];
                    to = dto.TimeRange[1];
                }
            }

            string numbers = dto.Numbers;
            numbers = numbers.Replace("\r\n", ",").Replace("\n", ",").Replace("\r", ",");
            numbers = FormatIdStr(numbers);//必须进行格式化处理，范围帐单处理时，会使用该字段进行联表查询

            Finance_OrderQuerySetting entity = new Finance_OrderQuerySetting
            {
                Name = dto.Name,
                DeliveryChannelIds = deliveryChannelIds,
                PublicChannelIds = publicChannelIds,
                CustomerIds = customerIds,
                TimeNode = dto.TimeNode,
                NumberType = dto.NumberType,
                From = from,
                To = to,
                Numbers = numbers,
                Status = OrderRangeQueryStatus.未执行,
                NotCustomerIds=notcustomerIds,
            };
            return entity;
        }

        public async Task<int> AddSetting(OrderRangeQuerySettingDto dto)
        {
            await CheckSave(dto, true);

            var entity = CreateEntity(dto);
            var id = await _orderQuerySettingRepository.InsertAsync(entity);
            return (int)id;
        }

        private async Task CheckSave(OrderRangeQuerySettingDto dto, bool isCreateNew)
        {
            if (dto == null)
                throw new Exception("参数不能为空");
            if (string.IsNullOrWhiteSpace(dto.Name))
                throw new Exception("名称不能为空");

            if (isCreateNew)
            {
                var sameNameExists = await _orderQuerySettingRepository.IsExistsAsync(i => i.Name == dto.Name && i.Deleted == false);
                if (sameNameExists == true)
                {
                    throw new Exception($"已存在同名的方案:【{dto.Name}】");
                }
            }
            else
            {
                if (dto.Id <= 0)
                    throw new Exception("ID不能为空");


                var sameNameExists = await _orderQuerySettingRepository.IsExistsAsync(i => i.Name == dto.Name && i.Id != dto.Id && i.Deleted == false);
                if (sameNameExists == true)
                {
                    throw new Exception($"已存在同名的方案:【{dto.Name}】");
                }
            }
        }

        public async Task<List<OrderRangeQuerySettingItem>> SearchSettingByName(string name)
        {
            List<Finance_OrderQuerySetting> settings = new List<Finance_OrderQuerySetting>();
            if (string.IsNullOrEmpty(name))
            {
                var entities = await _orderQuerySettingRepository.QueryAsync(i => i.Deleted == false);
                settings.AddRange(entities);

            }
            else
            {
                var entities = await _orderQuerySettingRepository.SearchSettingByName(name);
                if (entities?.Any() == true)
                {
                    settings.AddRange(entities);

                }
            }
            var items = settings.Select(s => new OrderRangeQuerySettingItem { Id = s.Id, Name = s.Name, value = s.Name }).ToList();
            return items;
        }

        public async Task<Finance_OrderQuerySetting> InsertOrderRange(int id)
        {
            var setting = await _orderQuerySettingRepository.GetAsync(id);
            if (setting == null)
                throw new Exception($"订单范围设置不存在,Id:{id}");

            int count = await InsertOrderRangeLocal(setting);
            return setting;
        }

        public async Task<int> InsertOrderRange(Finance_OrderQuerySetting setting)
        {
            var clearOk = await _OrderRangeSearchResultRepository.DeleteAsync(i => i.RangeSettingId == setting.Id);

            Finance_OrderRangeSearchResult orderRangeResult = new Finance_OrderRangeSearchResult();


            bool insertOk = false;
            int totalCount = 0;
            try
            {
                var result = await QueryOrderFromApi(setting);

                if (result?.Any() == true)
                {
                    totalCount = result.Count();
                    DateTime now = DateTime.Now;
                    var orderResult = result.Select(i => new Finance_OrderRangeSearchResult
                    {
                        CreateTime = now,
                        CreateUserName = "admin",
                        OrderId = i.OrderId,
                        RangeSettingId = setting.Id,
                        SystemNo = i.SystemNo,
                        BoxNumber = i.BoxNumber,
                        ChannelId = i.ChannelId,
                        PublicChannelId = i.PublicChannelId,
                        CreateOrderChannelId = i.CreateOrderChannelId,
                        CustomerId = i.CustomerId,
                        LadingBillNumber = i.LadingBillNumber,
                        OrderTime = i.OrderTime,
                        PackingTime = i.PackingTime,
                        ReceiveTime = i.ReceiveTime,
                        SendOutTime = i.SendOutTime,
                        TrackingNumber = i.TrackingNumber,
                        TransferNumber = i.TransferNumber,
                    }).ToList();


                    insertOk = await _OrderRangeSearchResultRepository.BulkToDBAsync(orderResult);
                }

                setting.IsExecuted = insertOk;
                setting.ExecutedTime = DateTime.Now;
                setting.TotalCount = totalCount;

                await _orderQuerySettingRepository.UpdateByConditionAsync(i => i.Id == setting.Id, new { IsExecuted = setting.IsExecuted, ExecutedTime = setting.ExecutedTime, TotalCount = setting.TotalCount, Status = OrderRangeQueryStatus.已执行, Message = string.Empty });
            }
            catch (Exception ex)
            {
                string msg = ex.Message;
                await _orderQuerySettingRepository.UpdateByConditionAsync(i => i.Id == setting.Id, new { IsExecuted = true, ExecutedTime = DateTime.Now, TotalCount = 0, Status = OrderRangeQueryStatus.失败, Message = msg });
            }
            return totalCount;
        }

        public async Task<IEnumerable<OrderRangeDto>> QueryOrderFromApi(Finance_OrderQuerySetting setting)
        {
            string api = "/api/OrderRangeSearch/QueryOrder";
            var result = await Request<IEnumerable<OrderRangeDto>>(api, setting);
            return result;
        }

        public async Task<T> Request<T>(string api, object param)
        {
            string token = "132A7468DE079C6CEB59F383A661E612";
            string host = "http://wms.xingyunyi.cn";

            RestClient client = new RestClient(host);
            RestRequest request = new RestRequest(api);

            Parameter tokenParam = new HeaderParameter("Token", token);
            request.AddParameter(tokenParam);
            request.AddJsonBody(param);

            var resp = await client.PostAsync<T>(request);
            return resp;
        }

        string FormatIdStr(string raw)
        {
            var ids = raw.Split(",", StringSplitOptions.RemoveEmptyEntries).OrderBy(x=>x);
            var idStr = string.Join(",", ids);
            return idStr;
        }

        public async Task<IEnumerable<OrderRangeQuerySettingItem>> GetAllSettings()
        {
            var result = await _orderQuerySettingRepository.QueryAsync(i => i.Deleted == false);
            var sortedResult = result?.OrderByDescending(i => i.CreateTime).ToList();
            return sortedResult?.Select(i => new OrderRangeQuerySettingItem { Id = i.Id, Name = i.Name, value = i.Name });
        }

        public async Task<PageResult<OrderRangeQuerySettingDto>> QueryPage(QueryModel queryModel)
        {
            var deletedField = queryModel.QueryParamer?.FirstOrDefault(i => i.Filed == "Deleted");
            if (deletedField == null)
            {
                QueryParamer deleteFieldParam = new QueryParamer { Filed = "Deleted", Method = "Eq", Value = "0" };
                if (queryModel.QueryParamer == null)
                    queryModel.QueryParamer = new List<QueryParamer>();
                queryModel.QueryParamer.Add(deleteFieldParam);
            }
            var entitiesPage = await _orderQuerySettingRepository.QueryPagerAsync(queryModel);
            var result = entitiesPage.MapperPager<Finance_OrderQuerySetting, OrderRangeQuerySettingDto>();
            return result;
        }

        private List<int> ToIdArray(string str)
        {
            List<int> result = new List<int>();
            if (!string.IsNullOrWhiteSpace(str))
            {
                var ids = str.Split(',');
                if (ids?.Any() == true)
                {
                    foreach (var idVal in ids)
                    {
                        if (int.TryParse(idVal, out int id))
                        {
                            result.Add(id);
                        }
                    }
                }
            }
            return result;
        }

        private string JoinString(IEnumerable<string> items)
        {
            string result = null;
            if (items?.Any() == true)
            {
                result = string.Join(',', items);
            }
            return result;
        }

        public async Task<OrderRangeQuerySettingDto> LoadRangeSetting(int id)
        {
            var setting = await _orderQuerySettingRepository.GetAsync(id);
            if (setting == null)
                throw new Exception($"订单范围搜索设置不存在,Id:{id}");

            OrderRangeQuerySettingDto dto = new OrderRangeQuerySettingDto
            {
                Name = setting.Name,
                Id = setting.Id,
                IsExecuted = setting.IsExecuted,
                Numbers = setting.Numbers,
                NumberType = setting.NumberType,
                TimeNode = setting.TimeNode,
            };

            dto.CustomerIds = ToIdArray(setting.CustomerIds);
            dto.PublicChannelIds = ToIdArray(setting.PublicChannelIds);
            dto.DeliveryChannelIds = ToIdArray(setting.DeliveryChannelIds);
            dto.CustomerNames = await GetCustomerCompanyNames(dto.CustomerIds);
            dto.PublicChannelNames = await GetPublicChannelNames(dto.PublicChannelIds);
            dto.DeliveryChannelNames = await GetDeliveryChannelNames(dto.DeliveryChannelIds);
            dto.TimeRange = new List<DateTime>();
            if (setting.From != null && setting.To != null)
            {
                dto.TimeRange.Add(setting.From.Value);
                dto.TimeRange.Add(setting.To.Value);
            }
            return dto;
        }

        private async Task<string> GetDeliveryChannelNames(List<int> deliveryChannelIds)
        {
            var channels = await _baseInfoService.GetAllDeliveryChannels();
            var names = channels?.Where(i => deliveryChannelIds.Contains(i.Id)).Select(i => i.Name);
            string result = JoinString(names);
            return result;
        }

        private async Task<string> GetPublicChannelNames(List<int> publicChannelIds)
        {
            var channels = await _baseInfoService.GetAllPublicChannels();
            var names = channels?.Where(i => publicChannelIds.Contains(i.Id)).Select(i => i.Name);
            string result = JoinString(names);
            return result;
        }

        private async Task<string> GetCustomerCompanyNames(List<int> customerIds)
        {
            var customers = await _baseInfoService.GetAllCustomers();
            var names = customers?.Where(i => customerIds.Contains(i.Id)).Select(i => i.CompanyName);
            string result = JoinString(names);
            return result;
        }

        public async Task<PageResult<OrderRangeResultDto>> QueryOrderResultPage(QueryModel queryModel)
        {
            if (queryModel == null)
                throw new Exception("参数类型不能为空");
            if (queryModel.QueryParamer?.Any() != true)
            {
                throw new Exception("必须指定至少一个搜索条件");
            }

            var numberType = queryModel.QueryParamer.FirstOrDefault(i => i.Filed == "NumberType");
            var numberField = queryModel.QueryParamer.FirstOrDefault(i => i.Filed == "Number");
            if (numberType != null)
            {
                queryModel.QueryParamer.Remove(numberType);
                if (numberField != null)
                {
                    string numberTypeName = "SystemNo";
                    if (numberType.Value == "0" || numberType.Value == "1")
                        numberTypeName = "SystemNo";
                    if (numberType.Value == "2")
                        numberTypeName = "TransferNumber";
                    if (numberType.Value == "3")
                        numberTypeName = "TrackingNumber";
                    if (numberType.Value == "4")
                        numberTypeName = "BoxNumber";
                    if (numberType.Value == "5")
                        numberTypeName = "LadingBillNumber";

                    numberField.Filed = numberTypeName;
                }
            }
            else
            {
                if (numberField != null)
                    queryModel.QueryParamer.Remove(numberField);
            }

            var entitiesPage = await _OrderRangeSearchResultRepository.QueryPagerAsync(queryModel);
            var dotPageResult = entitiesPage.MapperPager<Finance_OrderRangeSearchResult, OrderRangeResultDto>();
            if (dotPageResult?.Result?.Any() == true)
            {
                List<CustomerDto> allCustomers = null;
                var customerIds = dotPageResult.Result.Where(i => i.CustomerId.HasValue).Select(i => i.CustomerId.Value);
                if (customerIds?.Any() == true)
                {
                    allCustomers = await _baseInfoService.GetAllCustomers();
                }

                List<DeliveryChannelDto> allChannels = null;
                var channelIds = dotPageResult.Result.Where(i => i.ChannelId.HasValue).Select(i => i.ChannelId.Value);
                if (channelIds?.Any() == true)
                {
                    allChannels = await _baseInfoService.GetAllDeliveryChannels();
                }

                List<PublicChannelDto> allPublicChannels = null;
                var publicChannelsId = dotPageResult.Result.Where(i => i.PublicChannelId.HasValue).Select(i => i.PublicChannelId.Value);
                if (publicChannelsId?.Any() == true)
                {
                    allPublicChannels = await _baseInfoService.GetAllPublicChannels();
                }

                if (customerIds?.Any() == true || channelIds?.Any() == true || publicChannelsId?.Any() == true)
                {
                    foreach (var dto in dotPageResult.Result)
                    {
                        if (dto.CustomerId.HasValue)
                        {
                            var customer = allCustomers.FirstOrDefault(c => c.Id == dto.CustomerId.Value);
                            dto.CustomerName = customer?.CompanyName;
                        }

                        if (dto.ChannelId.HasValue)
                        {
                            var channel = allChannels.FirstOrDefault(c => c.Id == dto.ChannelId.Value);
                            dto.ChannelName = channel?.Name;
                        }

                        if (dto.PublicChannelId.HasValue)
                        {
                            var publicChannel = allPublicChannels.FirstOrDefault(c => c.Id == dto.PublicChannelId.Value);
                            dto.PublicChannelName = publicChannel?.Name;
                        }
                    }
                }
            }

            return dotPageResult;
        }

        public async Task<Finance_OrderQuerySetting> Execute(int id)
        {
            var setting = await InsertOrderRange(id);
            return setting;
        }

        public async Task<bool> Remove(int id)
        {
            var ok1 = await _orderQuerySettingRepository.UpdateByConditionAsync(s => s.Id == id, new { Deleted = true });
            return ok1;
        }

        public async Task<int> InsertOrderRangeLocal(Finance_OrderQuerySetting setting)
        {
            try
            {
                var clearOk = await _OrderRangeSearchResultRepository.DeleteAsync(i => i.RangeSettingId == setting.Id);

                string sql = @$" insert into Finance_OrderRangeSearchResult(RangeSettingId,OrderId,SystemNo,CreateUserName,CreateTime)
                            select {setting.Id} as RangeSettingId,0,o.SystemNo,'admin' as CreateUserName,getdate() as CreateTime
                            from Logistisc_Order o ";

                string where = "where 1=1 ";
                if (!string.IsNullOrWhiteSpace(setting.CustomerIds))
                {
                    var customerIdStr = FormatIdStr(setting.CustomerIds);
                    where += $" and o.CustomerId in ({customerIdStr})";
                }
                if (!string.IsNullOrWhiteSpace(setting.NotCustomerIds))
                {
                    var customerIdStr = FormatIdStr(setting.NotCustomerIds);
                    where += $" and o.CustomerId not in ({customerIdStr})";
                }
                if (!string.IsNullOrWhiteSpace(setting.PublicChannelIds))
                {
                    var publicChannelIdStr = FormatIdStr(setting.PublicChannelIds);
                    where += $" and o.PublicChannelId in ({publicChannelIdStr})";
                }
                if (!string.IsNullOrWhiteSpace(setting.DeliveryChannelIds))
                {
                    var deliveryChannelIdStr = FormatIdStr(setting.DeliveryChannelIds);
                    where += $" and o.OrderChannelUniqueId in ({deliveryChannelIdStr})";
                }
                if (setting.TimeNode != null && setting.From != null && setting.To != null)
                {
                    string timeNode = null;
                    if (setting.TimeNode == Model.Enums.TimeNode.下单时间)
                        timeNode = "o.OrderTime";
                    else if (setting.TimeNode == Model.Enums.TimeNode.揽收时间)
                        timeNode = "o.ReceiveTime";
                    else if (setting.TimeNode == Model.Enums.TimeNode.装箱时间)
                        timeNode = "o.PackingTime";
                    else if (setting.TimeNode == Model.Enums.TimeNode.发货时间)
                        timeNode = "o.SendOutTime";

                    string fromStr = setting.From.Value.ToString("yyyy-MM-dd HH:mm:ss");
                    string toStr = setting.To.Value.ToString("yyyy-MM-dd HH:mm:ss");

                    where += @$" and {timeNode} between '{fromStr}' and '{toStr}'";
                }
                if (!string.IsNullOrWhiteSpace(setting.Numbers))
                {
                    string numberType = null;
                    if (setting.NumberType == Model.Enums.NumberType.提单号)
                        numberType = "o.LadingBillNumber";
                    if (setting.NumberType == Model.Enums.NumberType.箱号)
                        numberType = "o.BoxNumber";
                    if (setting.NumberType == Model.Enums.NumberType.系统单号)
                        numberType = "o.SystemNo";
                    if (setting.NumberType == Model.Enums.NumberType.跟踪号)
                        numberType = "o.TrackingNumber";
                    if (setting.NumberType == Model.Enums.NumberType.转单号)
                        numberType = "o.TransferNumber";

                    var numbers = setting.Numbers.Split(",");
                    StringBuilder sb = new StringBuilder(2000);
                    foreach (var n in numbers)
                    {
                        sb.AppendFormat("\'{0}\',", n);
                    }
                    string numberStr = sb.ToString().TrimEnd(',');
                    where += @$" and {numberType} in({numberStr})";

                }
                sql += where;

                var result = await _OrderRangeSearchResultRepository.ExecuteSqlAsync(sql);


                setting.IsExecuted = true;
                setting.ExecutedTime = DateTime.Now;
                await _orderQuerySettingRepository.UpdateByConditionAsync(i => i.Id == setting.Id, new { IsExecuted = setting.IsExecuted, Status = OrderRangeQueryStatus.已执行, TotalCount= result, ExecutedTime = setting.ExecutedTime });
                return result;
            }
            catch(Exception ex)
            {
                string msg = ex.Message;
                await _orderQuerySettingRepository.UpdateByConditionAsync(i => i.Id == setting.Id, new { IsExecuted =false, Status = OrderRangeQueryStatus.失败,Message= msg, ExecutedTime = setting.ExecutedTime });
                return 0;
            }

        }

        public Task<Finance_OrderQuerySetting> GetSettings(int orderRangeSettingId)
        {
            var result = _orderQuerySettingRepository.GetAsync(orderRangeSettingId);
            return result;
        }
    }
}
