﻿using Microsoft.Extensions.Caching.Distributed;
using NPOI.SS.UserModel;
using NPOI.XSSF.UserModel;
using SMP.Data.Compute;
using SMP.Data.Report;
using SMP.Model.Report;
using SMP.Service.BaseInfo;
using SMP.Service.Report.Dto;
using SMP.Service.ThirdPartyRequest;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XYY.Common.Standard;
using XYY.Core.Standard.ExcelHelper;

namespace SMP.Service.Report
{
    public interface IGeneralCostReportService
    {
        Task<PageResult<GeneralCostDto>> QueryGeneralCostReport(QueryModel queryModel);
        Task<byte[]> DownGeneralCostReport(QueryModel param);
        Task<CostBatchDto> LoadCostDetail(LoadCostDetailParam param);
    }

    public class GeneralCostReportService : IGeneralCostReportService
    {
        IView_GeneralCostRepository _generalCostRepository;
        IBaseInfoService _baseInfoService;
        ICompute_ExpenseAmountRepository _expenseAmountRepository;
        ICompute_ExpenseRepository _expenseRepository;
        IBranchCompanyService _branchCompanyConnection;

        public GeneralCostReportService(IView_GeneralCostRepository generalCostRepository, IBaseInfoService baseInfoService, ICompute_ExpenseAmountRepository expenseAmountRepository, ICompute_ExpenseRepository expenseRepository, IBranchCompanyService branchCompanyConnection)
        {
            _generalCostRepository = generalCostRepository;
            _baseInfoService = baseInfoService;
            _expenseAmountRepository = expenseAmountRepository;
            _expenseRepository = expenseRepository;
            _branchCompanyConnection = branchCompanyConnection;
        }

        public async Task<byte[]> DownGeneralCostReport(QueryModel queryModel)
        {
            queryModel.PageIndex = 1;
            queryModel.PageSize = int.MaxValue;
            var pageResult = await QueryGeneralCostReport(queryModel);
            var dtoList = pageResult?.Result?.ToList();
            if (dtoList?.Any() == true)
            {
                IWorkbook wk = new XSSFWorkbook();
                ISheet sheet = wk.CreateSheet("Sheet1");


                OPNPOIRow<GeneralCostDto> row = new OPNPOIRow<GeneralCostDto>(new GeneralCostDto(), sheet.CreateRow(0));
                row.Cell(x => x.TrackingNumber)
                 .Cell(x => x.SystemNo)
                 .Cell(x => x.BoxNumber)
                .Cell(x => x.LadingBillNumber)
                .Cell(x => x.CustomerName)
                .Cell(x => x.PublicChannelName)
                .Cell(x => x.DeliveryChannelName)
                .Cell(x => x.OrderTime)
                .Cell(x => x.ReceiveTime)
                .Cell(x => x.PackingTime)
                .Cell(x => x.SendOutTime)
                .Cell(x => x.CompanyPossessName)
                .Cell(x => x.EstimateSettleAmount)
                .Cell(x => x.BillSettleAmount);


                for (int j = 0; j < dtoList.Count(); j++)
                {
                    row.Serialize(dtoList[j], sheet.CreateRow(j + 1));
                }

                return row.WBToByte(wk);
            }
            else return null;
        }

        public async Task<CostBatchDto> LoadCostDetail(LoadCostDetailParam param)
        {
            if (string.IsNullOrWhiteSpace(param?.SystemNo))
                throw new Exception("参数不能为空");

            var amounts = await _expenseAmountRepository.QueryAsync(i => i.Number == param.SystemNo && i.IsDelete == false);
            CostBatchDto result = new CostBatchDto();
            if (amounts?.Any() == true)
            {
                var details = amounts.Select(i => new ExpenseAmountItem { AmountId = i.Id, CostType = i.CostType, ExpenseId = i.ExpenseId, SettlementAmount = i.SettlementAmount }).ToList();


                var expenseIds = details.Select(i => i.ExpenseId);

                var allExpenses = await _expenseRepository.QueryAsync(i => i.Id.In(expenseIds));

                foreach (var d in details)
                {
                    var expense = allExpenses.FirstOrDefault(e => e.Id == d.ExpenseId);
                    d.ExpenseName = expense?.Name;
                }

                result.EstimateCost = details.Where(i => i.CostType == Model.Enums.CostType.预估成本).OrderBy(i => i.ExpenseId).ToList();
                result.BillCost = details.Where(i => i.CostType == Model.Enums.CostType.账单成本).OrderBy(i => i.ExpenseId).ToList();
            }
            return result;
        }

        public async Task<PageResult<GeneralCostDto>> QueryGeneralCostReport(QueryModel queryModel)
        {
            if (queryModel == null)
            {
                throw new ArgumentNullException("参数不能为空");
            }
            PreHandleQueryModel(queryModel);

            var pageResult = await _generalCostRepository.QueryPagerAsync(queryModel);
            var dtoPage = pageResult.MapperPager<View_GeneralCost, GeneralCostDto>();

            if (dtoPage?.Result?.Any() == true)
            {
                var systemNos = dtoPage.Result.Select(i => i.SystemNo);
                var branchCompanyOrderAmountsTask = _branchCompanyConnection.GetBranchCompanyOrderAmount(systemNos);

                var customers = await _baseInfoService.CacheGetAllCustomers();
                var deliveryChannels = await _baseInfoService.CacheGetAllDeliveryChannels();
                var publicChannels = await _baseInfoService.CacheGetAllPublicChannels();

                var branchCompanyOrderAmountsResp = await branchCompanyOrderAmountsTask;
                var branchCompanyOrderAmounts = branchCompanyOrderAmountsResp.Data;

                foreach (var dto in dtoPage.Result)
                {
                    var customer = customers?.FirstOrDefault(i => i.Id == dto.CustomerId);
                    dto.CustomerName = customer?.CompanyName;

                    var publicChannel = publicChannels?.FirstOrDefault(i => i.Id.ToString() == dto.PublicChannelId);
                    dto.PublicChannelName = publicChannel?.Name;

                    var deliveryChannel = deliveryChannels?.FirstOrDefault(i => i.Id.ToString() == dto.OrderChannelUniqueId);
                    dto.DeliveryChannelName = deliveryChannel?.Name;

                    var bc = branchCompanyOrderAmounts?.FirstOrDefault(d => d.SystemNo == dto.SystemNo);
                    dto.CustomerAmount = bc?.CustomerAmount;
                    dto.BranchCompanyAmount = bc?.BranchCompanyAmount;
                    dto.BranchCompanyProfit = bc?.BranchCompanyProfit;

                    dto.HeadCompanyEstimateProfit = bc?.CustomerAmount - dto.EstimateSettleAmount;
                    dto.HeadCompanyActualProfit = bc?.CustomerAmount - dto.BillSettleAmount;
                }
            }

            return dtoPage;
        }

        private void PreHandleQueryModel(QueryModel queryModel)
        {
            if (queryModel.QueryParamer?.Any() == true)
            {
                var numberTypeP = queryModel.QueryParamer.FirstOrDefault(i => i.Filed == "NumberType");
                var numberFieldP = queryModel.QueryParamer.FirstOrDefault(i => i.Filed == "Number");
                if (numberTypeP != null)
                {
                    queryModel.QueryParamer.Remove(numberTypeP);
                    if (numberFieldP != null)
                    {
                        string numberTypeName = "SystemNo";
                        if (numberTypeP.Value == "0" || numberTypeP.Value == "1")
                            numberTypeName = "SystemNo";
                        if (numberTypeP.Value == "2")
                            numberTypeName = "TransferNumber";
                        if (numberTypeP.Value == "3")
                            numberTypeName = "TrackingNumber";
                        if (numberTypeP.Value == "4")
                            numberTypeName = "BoxNumber";
                        if (numberTypeP.Value == "5")
                            numberTypeName = "LadingBillNumber";

                        numberFieldP.Filed = numberTypeName;
                    }
                }
                else
                {
                    if (numberFieldP != null)
                        queryModel.QueryParamer.Remove(numberFieldP);
                }

                var timeNodeP = queryModel.QueryParamer.FirstOrDefault(i => i.Filed == "TimeNode");
                var timeFromP = queryModel.QueryParamer.FirstOrDefault(i => i.Filed == "From");
                var timeToP = queryModel.QueryParamer.FirstOrDefault(i => i.Filed == "To");

                if (timeNodeP != null)
                {
                    queryModel.QueryParamer.Remove(timeNodeP);
                }
                if (timeFromP != null)
                {
                    queryModel.QueryParamer.Remove(timeFromP);
                }
                if (timeToP != null)
                {
                    queryModel.QueryParamer.Remove(timeToP);
                }

                if (timeNodeP != null && timeFromP != null && timeToP != null)
                {
                    string timeNodeVal = timeNodeP.Value;
                    string timeNodeField = "OrderTime";
                    if (timeNodeVal == "1")
                        timeNodeField = "ReceiveTime";
                    else if (timeNodeVal == "2")
                        timeNodeField = "PackingTime";
                    else if (timeNodeVal == "3")
                        timeNodeField = "SendOutTime";

                    var timeFrom = timeFromP.Value;
                    var timeTo = timeToP.Value;

                    DateTime.TryParse(timeFrom, out DateTime f);
                    DateTime.TryParse(timeTo, out DateTime t);

                    QueryParamer newTimeFrom = new QueryParamer { Filed = timeNodeField, Method = "GrE", Value = timeFrom };
                    QueryParamer newTimeTo = new QueryParamer { Filed = timeNodeField, Method = "LeE", Value = timeTo };
                    queryModel.QueryParamer.Add(newTimeFrom);
                    queryModel.QueryParamer.Add(newTimeTo);
                }
            }
        }
    }
}
