﻿using Microsoft.Extensions.Caching.Distributed;
using NPOI.SS.UserModel;
using NPOI.XSSF.UserModel;
using SMP.Data.Compute;
using SMP.Data.Report;
using SMP.Model.Report;
using SMP.Service.BaseInfo;
using SMP.Service.Report.Dto;
using SMP.Service.ThirdPartyRequest;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XYY.Common.Standard;
using XYY.Core.Standard.ExcelHelper;

namespace SMP.Service.Report
{
    public interface IGeneralCostReportV2Service
    {
        Task<PageResult<GeneralCostV2Dto>> QueryGeneralCostReport(QueryModel queryModel);
        Task<byte[]> DownGeneralCostReport(QueryModel param);
        Task<CostBatchDto> LoadCostDetail(LoadCostDetailParam param);
    }

    public class GeneralCostReportV2Service : IGeneralCostReportV2Service
    {
        IView_GeneralCostV2Repository _generalCostRepository;
        IBaseInfoService _baseInfoService;
        ICompute_ExpenseAmountRepository _expenseAmountRepository;
        ICompute_ExpenseRepository _expenseRepository;
        IBranchCompanyService _branchCompanyConnection;

        public GeneralCostReportV2Service(IView_GeneralCostV2Repository generalCostRepository, IBaseInfoService baseInfoService, ICompute_ExpenseAmountRepository expenseAmountRepository, ICompute_ExpenseRepository expenseRepository, IBranchCompanyService branchCompanyConnection)
        {
            _generalCostRepository = generalCostRepository;
            _baseInfoService = baseInfoService;
            _expenseAmountRepository = expenseAmountRepository;
            _expenseRepository = expenseRepository;
            _branchCompanyConnection = branchCompanyConnection;
        }

        public async Task<byte[]> DownGeneralCostReport(QueryModel queryModel)
        {
            queryModel.PageIndex = 1;
            queryModel.PageSize = int.MaxValue;
            var pageResult = await QueryGeneralCostReport(queryModel);
            var dtoList = pageResult?.Result?.ToList();
            if (dtoList?.Any() == true)
            {
                IWorkbook wk = new XSSFWorkbook();
                ISheet sheet = wk.CreateSheet("Sheet1");


                OPNPOIRow<GeneralCostV2Dto> row = new OPNPOIRow<GeneralCostV2Dto>(new GeneralCostV2Dto(), sheet.CreateRow(0));
                row.Cell(x => x.TrackingNumber)
                 .Cell(x => x.SystemNo)
                 .Cell(x => x.BoxNumber)
                .Cell(x => x.LadingBillNumber)
                .Cell(x => x.CustomerName)
                .Cell(x => x.PublicChannelName)
                .Cell(x => x.DeliveryChannelName)
                .Cell(x => x.OrderTime)
                .Cell(x => x.ReceiveTime)
                .Cell(x => x.PackingTime)
                .Cell(x => x.SendOutTime)
                .Cell(x => x.TotalEstimateCost)
                .Cell(x => x.TotalBillCost);


                for (int j = 0; j < dtoList.Count(); j++)
                {
                    row.Serialize(dtoList[j], sheet.CreateRow(j + 1));
                }

                return row.WBToByte(wk);
            }
            else return null;
        }

        public async Task<CostBatchDto> LoadCostDetail(LoadCostDetailParam param)
        {
            if (string.IsNullOrWhiteSpace(param?.SystemNo))
                throw new Exception("参数不能为空");

            var amounts = await _expenseAmountRepository.QueryAsync(i => i.Number == param.SystemNo && i.IsDelete == false);
            CostBatchDto result = new CostBatchDto();
            if (amounts?.Any() == true)
            {
                var details = amounts.Select(i => new ExpenseAmountItem { AmountId = i.Id, CostType = i.CostType, ExpenseId = i.ExpenseId, SettlementAmount = i.SettlementAmount }).ToList();


                var expenseIds = details.Select(i => i.ExpenseId);

                var allExpenses = await _expenseRepository.QueryAsync(i => i.Id.In(expenseIds));

                foreach (var d in details)
                {
                    var expense = allExpenses.FirstOrDefault(e => e.Id == d.ExpenseId);
                    d.ExpenseName = expense?.Name;
                }

                result.EstimateCost = details.Where(i => i.CostType == Model.Enums.CostType.预估成本).OrderBy(i => i.ExpenseId).ToList();
                result.BillCost = details.Where(i => i.CostType == Model.Enums.CostType.账单成本).OrderBy(i => i.ExpenseId).ToList();
            }
            return result;
        }

        public async Task<PageResult<GeneralCostV2Dto>> QueryGeneralCostReport(QueryModel queryModel)
        {
            if (queryModel == null)
            {
                throw new ArgumentNullException("参数不能为空");
            }
            ReportQueryParamPreHandle.PreHandleQueryModel(ref queryModel);

            var pageResult = await _generalCostRepository.QueryPagerAsync(queryModel);
            var dtoPage = pageResult.MapperPager<View_GeneralCostV2, GeneralCostV2Dto>();

            if (dtoPage?.Result?.Any() == true)
            {
                var systemNos = dtoPage.Result.Select(i => i.SystemNo);
                var branchCompanyOrderAmountsTask = _branchCompanyConnection.GetBranchCompanyOrderAmount(systemNos);

                var customers = await _baseInfoService.CacheGetAllCustomers();
                var deliveryChannels = await _baseInfoService.CacheGetAllDeliveryChannels();
                var publicChannels = await _baseInfoService.CacheGetAllPublicChannels();

                var branchCompanyOrderAmountsResp = await branchCompanyOrderAmountsTask;
                var branchCompanyOrderAmounts = branchCompanyOrderAmountsResp.Data;

                foreach (var dto in dtoPage.Result)
                {
                    var customer = customers?.FirstOrDefault(i => i.Id == dto.CustomerId);
                    dto.CustomerName = customer?.CompanyName;

                    var publicChannel = publicChannels?.FirstOrDefault(i => i.Id.ToString() == dto.PublicChannelId);
                    dto.PublicChannelName = publicChannel?.Name;

                    var deliveryChannel = deliveryChannels?.FirstOrDefault(i => i.Id.ToString() == dto.OrderChannelUniqueId);
                    dto.DeliveryChannelName = deliveryChannel?.Name;

                    var bc = branchCompanyOrderAmounts?.FirstOrDefault(d => d.SystemNo == dto.SystemNo);
                    dto.CustomerAmount = bc?.CustomerAmount;
                    dto.BranchCompanyAmount = bc?.BranchCompanyAmount;
                    dto.BranchCompanyProfit = bc?.BranchCompanyProfit;
                }
            }

            return dtoPage;
        }

       
    }
}
