﻿using SMP.Data.Compute;
using SMP.Data.Finance;
using SMP.Model.Compute;
using SMP.Model.Finance;
using SMP.Service.Dtos;
using SMP.Service.Reconcile.Dto;
using SMP.Service.ServiceBillExpenseMapping.Dto;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XYY.Common.Standard;

namespace SMP.Service.ServiceBillExpenseMapping
{
    public interface IServiceBillExpenseMappingService
    {
        Task<IEnumerable<NameIdDto>> GetAllMappingSolutions();
        Task<PageResult<ServiceBillExpenseMappingSolutionDto>> QuerySolutionPage(QueryModel queryModel);

        Task<bool> Remove(int id);
        Task<int> SaveMappingSolution(ServiceBillExpenseMappingSolutionDto dto);
        Task<ServiceBillDataNameExpense> GetBillDataNameAndExpenses(GetBillDataNameAndExpensesParam param);

        Task<ServiceBillExpenseMappingSolutionDto> LoadMappingSolution(LoadExpenseMappingSolutionParam param);
        Task<IEnumerable<NameIdDto>> GetServiceBillMappings(string billName);
    }

    public class ServiceBillExpenseMappingService : IServiceBillExpenseMappingService
    {
        IFinance_ServiceBillExpenseMappingRepository _mappingRepository;
        IFinance_ServiceBillExpenseMappingSolutionRepository _mappingSolutionRepository;
        IFinance_ServiceBillResolveSolutionRepository _serviceBillResolveSolutionRepository;
        IFinance_ServiceBillResolveResultRepository _serviceBillResolveResultRepository;
        ICompute_ExpenseRepository _expenseRepository;

        public ServiceBillExpenseMappingService(IFinance_ServiceBillExpenseMappingRepository mappingRepository,
            IFinance_ServiceBillExpenseMappingSolutionRepository mappingSolutionRepository,
            IFinance_ServiceBillResolveSolutionRepository serviceBillResolveSolutionRepository,
            IFinance_ServiceBillResolveResultRepository serviceBillResolveResultRepository,
            ICompute_ExpenseRepository expenseRepository
            )
        {
            _mappingRepository = mappingRepository;
            _mappingSolutionRepository = mappingSolutionRepository;
            _serviceBillResolveSolutionRepository = serviceBillResolveSolutionRepository;
            _serviceBillResolveResultRepository = serviceBillResolveResultRepository;
            _expenseRepository = expenseRepository;
        }

        public async Task<IEnumerable<NameIdDto>> GetAllMappingSolutions()
        {
            List<NameIdDto> result = new List<NameIdDto>();
            var mappingSolutions = await _mappingSolutionRepository.QueryAsync(i => i.Deleted == false);
            if (mappingSolutions?.Any() == true)
            {
                result.AddRange(mappingSolutions.Select(i => new NameIdDto { Id = i.Id, Name = i.Name }));
                return result;
            }
            else return result;
        }



        public async Task<PageResult<ServiceBillExpenseMappingSolutionDto>> QuerySolutionPage(QueryModel queryModel)
        {
            if (queryModel.QueryParamer == null)
            {
                queryModel.QueryParamer = new List<QueryParamer>();
            }
            if (!queryModel.QueryParamer.Any(i => i.Filed == "Deleted"))
            {
                queryModel.QueryParamer.Add(new QueryParamer { Filed = "Deleted", Method = "NotEq", Value = "1" });
            }
            var solutionEntityPage = await _mappingSolutionRepository.QueryPagerAsync(queryModel);
            var result = solutionEntityPage.MapperPager<Finance_ServiceBillExpenseMappingSolution, ServiceBillExpenseMappingSolutionDto>();

            return result;
        }

        public async Task<bool> Remove(int id)
        {
            var ok1 = await _mappingSolutionRepository.UpdateByConditionAsync(s => s.Id == id, new { Deleted = true });
            var ok2 = await _mappingRepository.UpdateByConditionAsync(s => s.MappingSolutionId == id, new { Deleted = true });
            return ok1 && ok2;
        }

        public async Task<int> SaveMappingSolution(ServiceBillExpenseMappingSolutionDto dto)
        {
            if (dto == null)
                throw new Exception("保存方案参数不能为空");
            if (dto.Id <= 0)
                return await CreateMappingSolution(dto);
            else return await UpdateMappingSolution(dto);
        }

        public async Task<int> UpdateMappingSolution(ServiceBillExpenseMappingSolutionDto dto)
        {
            ValidateSolutionDto(dto, true);

            int solutionId = dto.Id;
            var solution = await _mappingSolutionRepository.GetAsync(solutionId);
            if (solution == null)
                throw new Exception($"费用映射方案不存在,Id:{dto.Id}");

            var serviceBillResolveSolution = await _serviceBillResolveSolutionRepository.GetAsync(dto.ResolveSolutionId);
            if (serviceBillResolveSolution == null)
            {
                throw new Exception($"账单解析方案不存在,Id:{dto.ResolveSolutionId}");
            }

            solution.Name = dto.Name;
            solution.Description = dto.Description;
            solution.ResolveSolutionId = dto.ResolveSolutionId;
            solution.Enabled = dto.Enabled;
            var param = new
            {
                Name = solution.Name,
                Description = solution.Description,
                ResolveSolutionId = solution.ResolveSolutionId,
                BillTypeCode = serviceBillResolveSolution.BillTypeCode,
                ServiceProviderId = serviceBillResolveSolution.ServiceProviderId,
                Enabled = solution.Enabled
            };
            await _mappingSolutionRepository.UpdateByConditionAsync(i => i.Id == dto.Id, param);

            var mappings = dto.Mappings.CopyList<Finance_ServiceBillExpenseMapping>().ToList();

            mappings.ForEach(r =>
            {
                if (r.MappingSolutionId <= 0) r.MappingSolutionId = solutionId;
                if (r.CreateTime == null) r.CreateTime = DateTime.Now;
                if (r.BillTypeCode == null) r.BillTypeCode = solution.BillTypeCode;
                if (r.ServiceProviderId <= 0) r.ServiceProviderId = solution.ServiceProviderId;
                r.UpdateTime = DateTime.Now;
            });


            await _mappingRepository.DeleteAsync(r => r.MappingSolutionId == solutionId);
            await _mappingRepository.BulkToDBAsync(mappings);

            return solutionId;
        }

        public async Task<int> CreateMappingSolution(ServiceBillExpenseMappingSolutionDto dto)
        {
            ValidateSolutionDto(dto, false);
            var exists = await _mappingSolutionRepository.QueryAsync(i => i.Name == dto.Name && i.Deleted == false);
            if (exists?.Any() == true)
            {
                throw new Exception($"已存在同名的对账方案:{dto.Name}");
            }
            var serviceBillResolveSolution = await _serviceBillResolveSolutionRepository.GetAsync(dto.ResolveSolutionId);
            if (serviceBillResolveSolution == null)
            {
                throw new Exception($"账单解析方案不存在,Id:{dto.ResolveSolutionId}");
            }


            int mappingSolutionId = dto.Id;

            var now = DateTime.Now;
            Finance_ServiceBillExpenseMappingSolution solution = new Finance_ServiceBillExpenseMappingSolution
            {
                Name = dto.Name,
                Description = dto.Description,
                CreateTime = now,
                Deleted = false,
                BillTypeCode = serviceBillResolveSolution.BillTypeCode,
                ServiceProviderId = serviceBillResolveSolution.ServiceProviderId,
                Enabled = dto.Enabled,
                ResolveSolutionId = dto.ResolveSolutionId
            };


            var mappings = dto.Mappings.CopyList<Finance_ServiceBillExpenseMapping>().ToList();

            long solutionId = await _mappingSolutionRepository.InsertAsync(solution);
            if (solutionId <= 0)
            {
                throw new Exception("保存映射方案失败");
            }
            else
            {
                mappings.ForEach(r =>
                {
                    r.CreateTime = now;
                    r.MappingSolutionId = (int)solutionId;
                    r.BillTypeCode = serviceBillResolveSolution.BillTypeCode;
                    r.ServiceProviderId = serviceBillResolveSolution.ServiceProviderId;
                });
            }
            await _mappingRepository.BulkToDBAsync(mappings);

            return (int)solutionId;
        }

        private void ValidateSolutionDto(ServiceBillExpenseMappingSolutionDto dto, bool checkId)
        {
            if (dto == null)
                throw new Exception("参数不能为空");
            if (dto.Mappings?.Any() != true)
                throw new Exception("方案规则不能为空");

            if (checkId && dto.Id <= 0)
                throw new Exception("方案Id无效");

            if (dto.ResolveSolutionId <= 0)
                throw new Exception("账单解析方案Id无效");

            var duplicateBillDataNames = dto.Mappings.GroupBy(m => m.BillDataName).Where(g => g.Count() > 1);
            if (duplicateBillDataNames?.Any() == true)
            {
                var names = duplicateBillDataNames.Select(g => g.Key);
                var namesStr = string.Join(",", names);
                throw new Exception($"账单费用不能重复:{namesStr}");
            }
            foreach(var m in dto.Mappings)
            {
                if(m.ExpenseId<=0)
                    throw new Exception("映射中的费用ID无效");
                if (m.ExpenseItemId <= 0)
                    throw new Exception("映射中的费项ID无效");
            }
        }

        public async Task<ServiceBillDataNameExpense> GetBillDataNameAndExpenses(GetBillDataNameAndExpensesParam param)
        {
            if (param == null || param.ResolveSolutionId <= 0)
                throw new Exception("参数不能为空");

            ServiceBillDataNameExpense result = new ServiceBillDataNameExpense();
            var billDataNames = await _serviceBillResolveResultRepository.GetResolveDataNames(param.ResolveSolutionId);


            result.BillDataNames = billDataNames;

            IEnumerable<View_ExpenseItemInfo> expense = await _expenseRepository.GetServiceProviderExpenseInfo(param.ResolveSolutionId);

            var expenseItemInfoDtos = expense.Select(i=>new ExpenseItemInfoDto { 
            ExpenseId=i.ExpenseId,
            ExpenseItemId=i.ExpenseItemId,
            Name=$"{i.ExpenseName}({i.ExpenseItemName})"
            });
            result.Expenses = expenseItemInfoDtos?.ToList();

            return result;
        }

        public async Task<ServiceBillExpenseMappingSolutionDto> LoadMappingSolution(LoadExpenseMappingSolutionParam param)
        {
            var solutions = await _mappingSolutionRepository.QueryAsync(i => i.Deleted == false && i.Id == param.Id);
            if (solutions?.Any() != true) return null;

            var solution = solutions.First();
            ServiceBillExpenseMappingSolutionDto result = new ServiceBillExpenseMappingSolutionDto
            {
                CreateTime = solution.CreateTime,
                Description = solution.Description,
                Name = solution.Name,
                ResolveSolutionId = solution.ResolveSolutionId,
                Enabled = solution.Enabled,
                Id = solution.Id,
            };

            var mappings = await _mappingRepository.QueryAsync(i => i.Deleted == false && i.MappingSolutionId == param.Id);
            if (mappings?.Any() == true)
            {
                IList<ServiceBillExpenseMappingDto> mappingDtos = mappings.CopyList<ServiceBillExpenseMappingDto>();
                result.Mappings = mappingDtos.ToList();

                var expenseIds = mappings.Select(i => i.ExpenseId);

                var expenses = await _expenseRepository.QueryAsync(e => e.Id.In(expenseIds));
                if (expenses?.Any() == true)
                {
                    foreach (var m in result.Mappings)
                    {
                        var expense = expenses.FirstOrDefault(i => i.Id == m.ExpenseId);
                        m.SystemDataName = expense?.Name;
                    }
                }
            }


            return result;
        }

        public async Task<IEnumerable<NameIdDto>> GetServiceBillMappings(string billName)
        {
            var result = new List<NameIdDto>();
            if (string.IsNullOrWhiteSpace(billName))
            {
                return await Task.FromResult(result);
            }
            IEnumerable<Finance_ServiceBillExpenseMappingSolution> solutions = await _mappingSolutionRepository.GetServiceBillMappings(billName);
            if (solutions?.Any() == true)
            {
                var items = solutions.Select(i => new NameIdDto { Id = i.Id, Name = i.Name });
                result.AddRange(items);
            }
            return result;
        }
    }
}
