﻿using NPOI.SS.UserModel;
using NPOI.SS.Util;
using NPOI.XSSF.UserModel;
using SMP.Model.Enums;
using SMP.Model.Finance;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace SMP.Service.ServiceBillResolve
{
    class FeeListModeResolver : ServiceBillResoverBase
    {
        public override string ResolverName { get => "费用列表解析器"; }

        public override List<ServiceBillResolveResult> ResolveRule(IWorkbook workbook, ISheet sheet, Finance_ServiceBillResolveSolutionRule rule)
        {
            if (workbook == null)
                throw new Exception("工作簿workbook不能为空");
            if (sheet == null)
                throw new Exception("表单Sheet不能为空");

            if (rule.ListingType != BillListingStyle.FeeList)
                throw new Exception($"当前解析规则{rule.ListingType}与解析器{ResolverName}不匹配");
            if (string.IsNullOrWhiteSpace(rule.FeeNameAddressPattern))
                throw new Exception("费用名称地址模式不能为空");
            if (string.IsNullOrWhiteSpace(rule.DataAddressPattern))
                throw new Exception("数值地址模式不能为空");
            if (rule.NumberType != NumberType.无 && string.IsNullOrWhiteSpace(rule.NumberAddressPattern))
                throw new Exception("单号地址模式不能为空");


            var feeNameAp = base.AnalyzeAddressPattern(sheet, rule.FeeNameAddressPattern);


            AddressPatternInfo billTimeAp = null;
            if (!rule.SetBillTimeOnResolve)
            {
                billTimeAp = base.AnalyzeAddressPattern(sheet, rule.BillTimeAddressPattern);
            }


            AddressPatternInfo numberAp = null;
            if (rule.NumberType != NumberType.无)
            {
                numberAp = base.AnalyzeAddressPattern(sheet, rule.NumberAddressPattern);
            }


            string numberRegex = rule.NumberResolveRegx;
            if (string.IsNullOrWhiteSpace(numberRegex) && rule.NumberType != NumberType.无)
                numberRegex = GetDefaultNumberRegex(rule.NumberType);

            string number = null;
            if (rule.NumberType == NumberType.无)//无订单号也要赋值一个字符串，避免数据NULL的问题
            {
                number = "-";
            }
            else
            {
                number = GetFirstStringValue(workbook, sheet, numberAp, numberRegex);
                if (string.IsNullOrEmpty(number))
                    throw new Exception("不能找到单号");
            }

            var feeAp = base.AnalyzeAddressPattern(sheet, rule.DataAddressPattern);

            List<IRow> feeRows = GetRows(sheet, feeNameAp);

            List<ServiceBillResolveResult> result = new List<ServiceBillResolveResult>();
            if (feeRows?.Any() == true)
            {
                short feeNameColIndex = GetColumnIndex(feeNameAp);
                short feeColIndex = GetColumnIndex(feeAp);
                short billTimeColIndex = 0;
                if (!rule.SetBillTimeOnResolve)
                {
                    billTimeColIndex = GetColumnIndex(billTimeAp);
                }

                DateTime? fixedBillTime = null;

                foreach (var fr in feeRows)
                {
                    var feeNameCell = fr.GetCell(feeNameColIndex);

                    bool cellMergeChanged = CellMergeChanged(feeNameCell.IsMergedCell);
                    if (cellMergeChanged)
                    {
                        break;
                    }
                    
                    bool feeNameOk = TryGetStringValue(workbook, feeNameCell, out string feeName);
                    if (feeNameOk)
                    {
                        var feeCell = fr.GetCell(feeColIndex);
                        bool feeOk = TryGetNumericValue(workbook, feeCell, rule.DataUseRegx, out decimal fee, out string currency);
                        if (!feeOk)
                            fee = 0;


                        DateTime? billTime = null;
                        if (!rule.SetBillTimeOnResolve && billTimeAp != null)
                        {
                            if (billTimeAp.StopSymbol != StopSymbol.CertainRow)
                            {
                                var billTimeCell = fr.GetCell(billTimeColIndex);
                                bool billTimeOk = TryGetDateTimeValue(workbook, billTimeCell, out billTime);
                                if (!billTimeOk)
                                    throw new Exception($"读取时间失败，时间定为模式:{rule.BillTimeAddressPattern},行:{billTimeCell.RowIndex + 1},列:{billTimeCell.ColumnIndex + 1}");
                            }
                            else
                            {
                                if (fixedBillTime == null)
                                {
                                   var billTimeCellRef =  GetCellReference(billTimeAp.ColumnName, billTimeAp.StartRowNumber);

                                    short billTimeCellIndex = GetColumnIndex(billTimeAp);
                                    var billTimeRowIndex = billTimeAp.StartRowNumber;
                                    var r = sheet.GetRow(billTimeCellRef.Row);
                                    if (r == null)
                                        throw new Exception($"读取时间失败,读取行:{billTimeRowIndex}");

                                    var billTimeCell = r.GetCell(billTimeCellRef.Col);
                                    if (billTimeCell == null)
                                        throw new Exception($"读取时间失败,读取行:{billTimeRowIndex},列:{billTimeCellIndex}");

                                    bool billTimeOk = TryGetDateTimeValue(workbook, billTimeCell, out billTime);
                                    if (!billTimeOk)
                                        throw new Exception($"读取时间失败，时间定为模式:{rule.BillTimeAddressPattern},行:{billTimeCell.RowIndex + 1},列:{billTimeCell.ColumnIndex + 1}");
                                    else
                                        fixedBillTime = billTime;

                                }
                                else
                                {
                                    billTime = fixedBillTime.Value;
                                }

                            }
                        }

                        ServiceBillResolveResult rr = new ServiceBillResolveResult();
                        rr.BillDataName = feeName;
                        rr.Data = fee;
                        rr.Number = number;
                        rr.NumberType = rule.NumberType;
                        rr.RuleId = rule.Id;
                        rr.SolutionId = rule.SolutionId;
                        rr.Currency = string.IsNullOrEmpty(rule.Currency) ? currency : rule.Currency;
                        rr.BillTime = billTime;
                        result.Add(rr);
                    }
                }
            }
            return result;
        }


    }
}
