﻿using RestSharp;
using SMP.Data;
using SMP.Data.Finance;
using SMP.Model.Enums;
using SMP.Model.Finance;
using SMP.Model.Order;
using SMP.Service.Finance;
using SMP.Service.ServiceBillResolve.Dto;
using System;
using System.Collections.Generic;
using System.Dynamic;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;
using XYY.Common.Standard;

namespace SMP.Service.ServiceBillResolve
{
    public interface IServiceBillResolveService
    {
        Task<int> SaveResolveSolution(ServiceBillResolveSolutionDto dto);
        Task<int> UpdateResolveSolution(ServiceBillResolveSolutionDto dto);

        Task<int> CreateResolveSolution(ServiceBillResolveSolutionDto dto);
        Task<PageResult<ServiceBillResolveSolutionDto>> QuerySolutionPage(QueryModel queryModel);
        Task<ServiceBillResolveSolutionDto> LoadSolution(LoadSolutionParam param);

        Task<ServiceBillResolveApplyResult> ApplyResolve(ApplyResolveServiceBillParam param);
        Task<List<ServiceBillResolveSolutionDto>> GetAllSolutions(bool? enabled = null);
        Task<PageResult<ServiceBillResolveApplyDto>> QueryResoveApplyPage(QueryModel queryModel);

        Task<PageResult<ExpandoObject>> QueryResoveResultPage(QueryResoveResultPageParam param);
        Task<List<string>> GetBillDataNames(string billName);
        Task<IList<ServiceBillResolveSolutionRuleDto>> GetSolutionRules(GetSolutionRulesParam param);
        Task<IEnumerable<string>> SearchResolvedBillName(string name);
        Task<bool> CheckSetBillTimeOnResolve(CheckSetBillTimeOnResolveParam param);
        Task<bool> RemoveSolutionById(int id);
    }

    public class ServiceBillResolveService : IServiceBillResolveService
    {
        IFinance_ServiceBillResolveSolutionRepository _solutionRepository;
        IFinance_ServiceBillResolveSolutionRuleRepository _ruleRepository;
        IFinance_ServiceBillResolveApplyRepository _solutionApplyRepotitory;
        IServiceBillResolver _serviceBillResolver;
        IFinance_ServiceBillResolveResultRepository _ServiceBillResolveResultRepository;
        IFinance_OrderRangeSearchResultRepository _OrderRangeSearchResultRepository;
        ILogistisc_BillTypeService _billTypeService;
        IServiceProviderService _serviceProviderService;

        public ServiceBillResolveService(IFinance_ServiceBillResolveSolutionRepository solutionRepository, IFinance_ServiceBillResolveSolutionRuleRepository ruleRepository,
             IFinance_ServiceBillResolveApplyRepository solutionApplyRepotitory,
            IServiceBillResolver serviceBillResolver, IFinance_ServiceBillResolveResultRepository ServiceBillResolveResultRepository,
            IFinance_OrderRangeSearchResultRepository orderRangeSearchResultRepository, ILogistisc_BillTypeService billTypeService, IServiceProviderService serviceProviderService)
        {
            _solutionRepository = solutionRepository;
            _ruleRepository = ruleRepository;
            _solutionApplyRepotitory = solutionApplyRepotitory;
            _serviceBillResolver = serviceBillResolver;
            _ServiceBillResolveResultRepository = ServiceBillResolveResultRepository;
            _OrderRangeSearchResultRepository = orderRangeSearchResultRepository;
            _billTypeService = billTypeService;
            _serviceProviderService = serviceProviderService;
        }

        public async Task<ServiceBillResolveSolutionDto> LoadSolution(LoadSolutionParam param)
        {
            int id = param.Id;
            QueryModel queryModel = new QueryModel { PageIndex = 1, PageSize = 1, QueryParamer = new List<QueryParamer> { new QueryParamer { Filed = "Id", Method = "Eq", Value = id.ToString() } } };
            var page = await QuerySolutionPage(queryModel);
            if (page?.Result?.Any() == true)
            {
                return page.Result.First();
            }
            else return null;
        }

        public async Task<PageResult<ServiceBillResolveSolutionDto>> QuerySolutionPage(QueryModel queryModel)
        {
            var solutionEntityPage = await _solutionRepository.QueryPagerAsync(queryModel);
            var result = solutionEntityPage.MapperPager<Finance_ServiceBillResolveSolution, ServiceBillResolveSolutionDto>();


            if (result?.Result?.Any() == true)
            {
                var solutionIds = result?.Result.Select(s => s.Id);
                var allRules = await _ruleRepository.QueryAsync(r => r.SolutionId.In(solutionIds));

                var allRuleDtos = allRules.CopyList<ServiceBillResolveSolutionRuleDto>();
                var billTypeCodes = result.Result.Select(i => i.BillTypeCode);

                var billSets = await _billTypeService.GetByCodes(billTypeCodes);

                var serviceProviderIds = result.Result.Select(i => i.ServiceProviderId);
                var allServiceProviders = await _serviceProviderService.GetProviderByids(serviceProviderIds);

                foreach (var solution in result.Result)
                {
                    int sid = solution.Id;
                    var rules = allRuleDtos.Where(r => r.SolutionId == sid).ToList();

                    solution.Rules = rules;

                    var billset = billSets?.FirstOrDefault(i => i.BillTypeCode == solution.BillTypeCode);
                    if (billset != null)
                    {
                        solution.BillTypeName = billset.BillTypeName;

                    }

                    var serviceProvider = allServiceProviders?.FirstOrDefault(i => i.Id == solution.ServiceProviderId);
                    if (serviceProvider != null)
                    {
                        solution.ServiceProviderName = serviceProvider.ServiceName;
                    }
                }
            }
            return result;
        }

        private void ValidateSolutionDto(ServiceBillResolveSolutionDto dto, bool checkId)
        {
            if (dto == null)
                throw new Exception("参数不能为空");

            if (string.IsNullOrWhiteSpace(dto.Name))
                throw new Exception("方案名称不能为空");

            if (dto.Rules?.Any() != true)
                throw new Exception("方案规则不能为空");

            int i = 0;
            foreach (var r in dto.Rules)
            {
                i++;
                if (string.IsNullOrWhiteSpace(r.RuleName))
                {
                    throw new Exception($"第{i}条规则:规则名称不能为空");
                }
                if (string.IsNullOrWhiteSpace(r.SheetPattern))
                    throw new Exception($"第{i}条规则:表单(Sheet)定位模式不能为空");
            }

            if (checkId && dto.Id <= 0)
                throw new Exception("方案Id无效");

            if (dto.ServiceProviderId <= 0)
                throw new Exception("服务商不能为空");

            if (string.IsNullOrWhiteSpace(dto.BillTypeCode))
                throw new Exception("账单类型不能为空");

        }

        public async Task<int> CreateResolveSolution(ServiceBillResolveSolutionDto dto)
        {
            ValidateSolutionDto(dto, false);
            var exists = await _solutionRepository.QueryAsync(i => i.Name == dto.Name);
            if (exists?.Any() == true)
            {
                throw new Exception($"已存在同名的方案:{dto.Name}");
            }
            var now = DateTime.Now;
            Finance_ServiceBillResolveSolution solution = new Finance_ServiceBillResolveSolution
            {
                Name = dto.Name,
                BillTypeCode = dto.BillTypeCode,
                ServiceProviderId = dto.ServiceProviderId,
                WeightDataName = dto.WeightDataName,
                Description = dto.Description,
                CreateTime = now,
                Enabled = dto.Enabled
            };

            var rules = dto.Rules.CopyList<Finance_ServiceBillResolveSolutionRule>().ToList();

            long solutionId = await _solutionRepository.InsertAsync(solution);
            if (solutionId <= 0)
            {
                throw new Exception("保存方案失败");
            }
            else
            {
                rules.ForEach(r =>
                {
                    r.CreateTime = now;
                    r.SolutionId = (int)solutionId;

                    if (r.SetBillTimeOnResolve)
                        r.BillTimeAddressPattern = null;
                });
            }
            await _ruleRepository.BulkToDBAsync(rules);

            return (int)solutionId;
        }

        public async Task<int> UpdateResolveSolution(ServiceBillResolveSolutionDto dto)
        {
            ValidateSolutionDto(dto, true);

            int solutionId = dto.Id;
            var solution = await _solutionRepository.GetAsync(solutionId);
            if (solution == null)
                throw new Exception($"账单解析方案不存在,Id:{dto.Id}");

            solution.Name = dto.Name;
            solution.Description = dto.Description;

            var update = new { Name = solution.Name, Description = solution.Description, ServiceProviderId = dto.ServiceProviderId, BillTypeCode = dto.BillTypeCode, WeightDataName = dto.WeightDataName, Enabled = dto.Enabled };
            await _solutionRepository.UpdateByConditionAsync(i => i.Id == dto.Id, update);

            var rules = dto.Rules.CopyList<Finance_ServiceBillResolveSolutionRule>().ToList();

            rules.ForEach(r =>
            {
                if (r.SolutionId <= 0) r.SolutionId = solutionId;
                if (r.CreateTime == null) r.CreateTime = DateTime.Now;
                if (r.SetBillTimeOnResolve) r.BillTimeAddressPattern = null;
            });


            await _ruleRepository.DeleteAsync(r => r.SolutionId == solutionId);
            await _ruleRepository.BulkToDBAsync(rules);

            return solutionId;
        }

        public async Task<int> SaveResolveSolution(ServiceBillResolveSolutionDto dto)
        {
            if (dto == null)
                throw new Exception("保存方案参数不能为空");
            if (dto.Id <= 0)
                return await CreateResolveSolution(dto);
            else return await UpdateResolveSolution(dto);
        }

        public async Task<Tuple<Finance_ServiceBillResolveSolution, List<Finance_ServiceBillResolveSolutionRule>>> GetAndCheckSolution(int solutionId)
        {
            var solution = await _solutionRepository.GetAsync(solutionId);
            if (solution == null)
            {
                throw new Exception($"找不到账单解析方案,Id:{solutionId}");
            }
            if (!solution.Enabled)
            {
                throw new Exception($"该解析方案已停用");
            }
            var rules = await _ruleRepository.QueryAsync(r => r.SolutionId == solutionId);
            if (rules?.Any() != true)
                throw new Exception($"方案下没有解析规则,方案ID:{solutionId}");

            return new Tuple<Finance_ServiceBillResolveSolution, List<Finance_ServiceBillResolveSolutionRule>>(solution, rules.ToList());
        }

        public async Task<ServiceBillResolveApplyResult> ApplyResolveBatch(ApplyResolveServiceBillParam param)
        {
            ValidateApplyParam(param);

            var solutionAndRules = await GetAndCheckSolution(param.SolutionId);
            var solution = solutionAndRules.Item1;
            var rules = solutionAndRules.Item2;

            IEnumerable<ServiceBillFileEntry> billFiles = await GetBillFileEntries(param.FileUrl);

            try
            {
                foreach (ServiceBillFileEntry fileEntry in billFiles)
                {

                    string billName = fileEntry.BillName;
                    await CheckBillNameResultExists(billName);

                    Stream file = fileEntry.File;
                    var result = await ApplyResolveCore(solution, rules, file, billName, param.BatchFileName, null, param.FileUrl, param.Description);
                }
                return new ServiceBillResolveApplyResult { Success = true };
            }
            catch
            {
                if (billFiles?.Any() == true)
                {
                    foreach (var f in billFiles)
                    {
                        if (f.File != null)
                        {
                            f.File.Dispose();
                        }
                    }
                }
                throw;
            }
        }

        private async Task<IEnumerable<ServiceBillFileEntry>> GetBillFileEntries(string fileUrl)
        {
            var zipFileData = await DownloadFile(fileUrl);

            string temFile = Path.GetTempFileName();
            File.WriteAllBytes(temFile, zipFileData);
            var tempPath = Path.Combine(Path.GetTempPath(), "service_bill", DateTime.Now.ToString("yyyyMMddHHmmss"));
            ZipHelper.UnZipPath(temFile, tempPath);
            var files = Directory.GetFiles(tempPath);

            List<ServiceBillFileEntry> fileEntries = new List<ServiceBillFileEntry>();
            if (files?.Any() == true)
            {
                foreach (var file in files)
                {
                    string ext = Path.GetExtension(file);
                    if (ext == null || !(ext.Contains("xls", StringComparison.OrdinalIgnoreCase) || ext.Contains("xlsx", StringComparison.OrdinalIgnoreCase) || ext.Contains("pdf", StringComparison.OrdinalIgnoreCase)))
                    {
                        throw new Exception("仅支持xls,xlsx,pdf文件");
                    }
                    else
                    {
                        string billName = Path.GetFileName(file);

                        Stream fileStream = File.OpenRead(file);
                        ServiceBillFileEntry ent = new ServiceBillFileEntry
                        {
                            File = fileStream,
                            BillName = billName,
                        };
                        fileEntries.Add(ent);
                    }
                }
            }
            return fileEntries;
        }

        public async Task<ServiceBillResolveApplyResult> ApplyResolve(ApplyResolveServiceBillParam param)
        {
            if (param.IsBatch)
            {
                var result = await ApplyResolveBatch(param);
                return result;
            }
            else
            {
                var result = await ApplyResolveOne(param);
                return result;
            }
        }

        private async Task<ServiceBillResolveApplyResult> ApplyResolveOne(ApplyResolveServiceBillParam param)
        {
            ValidateApplyParam(param);
            await CheckBillNameResultExists(param.BillName);

            var solutionAndRules = await GetAndCheckSolution(param.SolutionId);
            var solution = solutionAndRules.Item1;
            var rules = solutionAndRules.Item2;
            byte[] fileData = await DownloadFile(param.FileUrl);

            MemoryStream billFile = new MemoryStream(fileData);
            try
            {
                var result = await ApplyResolveCore(solution, rules.ToList(), billFile, param.BillName, param.BatchFileName, param.BillTime, param.FileUrl, param.Description);
                return result;
            }
            catch (Exception ex)
            {
                return new ServiceBillResolveApplyResult { Success = false, Message = ex.Message };
            }
            finally
            {
                billFile.Close();
                billFile.Dispose();
            }
        }

        private async Task<byte[]> DownloadFile(string fileUrl)
        {
            RestClient client = new RestClient(fileUrl);
            RestRequest req = new RestRequest { Method = Method.Get };
            var fileData = await client.DownloadDataAsync(req);
            if (fileData == null || fileData.Length == 0)
                throw new Exception($"下载到的账单无效");
            return fileData;
        }

        public async Task<ServiceBillResolveApplyResult> ApplyResolveCore(Finance_ServiceBillResolveSolution solution, List<Finance_ServiceBillResolveSolutionRule> rules, Stream billFile, string billName, string batchFileName, DateTime? billTime, string rawFileUrl, string description)
        {

            int serviceProviderId = solution.ServiceProviderId;
            string billTypeCode = solution.BillTypeCode;
            ServiceBillResolveApplyResult result = new ServiceBillResolveApplyResult { Success = true };

            Finance_ServiceBillResolveApply apply = CreateResolveSolutionApply(serviceProviderId, billTypeCode, solution.Id, rawFileUrl, billName, batchFileName, description);
            long applyId = await _solutionApplyRepotitory.InsertAsync(apply);
            int applyId32 = (int)applyId;
            ServiceBillResolveParam resolveParam = await CreateResolveParam(solution.Id, billTime, billFile, rules);

            List<ServiceBillResolveResult> resultResult = null;
            try
            {
                resultResult = _serviceBillResolver.Resolve(resolveParam);
            }
            catch (Exception ex)
            {
                result.Success = false;
                result.Message = ex.Message;
                await _solutionApplyRepotitory.UpdateByConditionAsync(i => i.Id == applyId32, new { Status = ResolveStatus.失败, Message = ex.Message });
                return result;
            }
            finally
            {
                if (billFile != null)
                {
                    billFile.Dispose();
                    billFile = null;
                }

            }

            var resultEntities = resultResult.CopyList<Finance_ServiceBillResolveResult>();

            await FixSystemNo(resultEntities);

            DateTime createTime = DateTime.Now;
            if (resultEntities?.Any() == true)
            {
                foreach (var e in resultEntities)
                {
                    e.ApplyId = applyId32;
                    e.CreateTime = createTime;
                    e.BillName = billName;
                    e.ServiceProviderId = serviceProviderId;
                    e.BillTypeCode = billTypeCode;

                    if (!string.IsNullOrWhiteSpace(solution.WeightDataName) && solution.WeightDataName.Contains(e.BillDataName))
                    {
                        e.DataType = ServiceBillDataType.重量;
                    }
                    else
                    {
                        e.DataType = ServiceBillDataType.金额;
                    }
                }
                await _ServiceBillResolveResultRepository.BulkToDBAsync(resultEntities.ToList());
                await _solutionApplyRepotitory.UpdateByConditionAsync(i => i.Id == applyId32, new { Status = ResolveStatus.成功, FinishTime = DateTime.Now });
            }
            else
            {
                await _solutionApplyRepotitory.UpdateByConditionAsync(i => i.Id == applyId32, new { Status = ResolveStatus.失败, Message = "解析完成，但解析不到任何内容", FinishTime = DateTime.Now });
            }
            return result;
        }

        private void ValidateSystemNo(IList<Finance_ServiceBillResolveResult> resultEntities)
        {
            foreach (var i in resultEntities)
            {
                if (i.NumberType == NumberType.系统单号 || i.NumberType == NumberType.跟踪号 || i.NumberType == NumberType.转单号)
                {
                    if (string.IsNullOrWhiteSpace(i.SystemNo))
                    {
                        throw new Exception($"【{i.Number}】对应的系统单号为空");
                    }
                }
            }
        }

        /// <summary>
        /// 补全SytemNo
        /// </summary>
        /// <param name="resultEntities"></param>
        private async Task FixSystemNo(IList<Finance_ServiceBillResolveResult> resultEntities)
        {
            var numberTypeGroups = resultEntities.GroupBy(i => i.NumberType);
            if (numberTypeGroups?.Any() == true)
            {
                foreach (var g in numberTypeGroups)
                {
                    if (g.Key == NumberType.系统单号)
                    {
                        foreach (var i in g)
                        {
                            i.SystemNo = i.Number;
                        }
                    }
                    else if (g.Key == NumberType.跟踪号 || g.Key == NumberType.转单号)
                    {
                        var trackingNumbers = g.Select(i => i.Number);
                        Dictionary<string, string> numberPairs = await GetOrderSystemNoByTransferNumber(trackingNumbers, g.Key);
                        if (numberPairs?.Any() == true)
                        {
                            foreach (var i in g)
                            {
                                if (numberPairs.TryGetValue(i.Number, out string? num))
                                {
                                    i.SystemNo = num;
                                }
                            }
                        }
                    }
                }
            }
        }

        private async Task<Dictionary<string, string>> GetOrderSystemNoByTransferNumber(IEnumerable<string> numbers, NumberType numberType)
        {
            if (numbers?.Any() != true) return null;

            int pageSize = 2000;
            int totalCount = numbers.Count();
            int pageCount = (int)Math.Ceiling(totalCount / (decimal)pageSize);

            List<OrderNumberInfo> numberInfos = new List<OrderNumberInfo>();
            for (int p = 0; p <= 0; p++)
            {
                var pageNumbers = numbers.Skip(0 * pageSize).Take(pageSize);

                IEnumerable<OrderNumberInfo> pageNumberInfo = null;
                if (numberType == NumberType.转单号)
                {
                    pageNumberInfo = await _OrderRangeSearchResultRepository.GetOrderSystemNoByTransferNumber(pageNumbers);
                }
                else if (numberType == NumberType.跟踪号)
                {
                    pageNumberInfo = await _OrderRangeSearchResultRepository.GetOrderSystemNoByTrackingNumberNumber(pageNumbers);
                }

                if (pageNumberInfo?.Any() == true)
                {
                    numberInfos.AddRange(pageNumberInfo);
                }
            }
            if (numberType == NumberType.转单号)
            {
                var dic = numberInfos.ToDictionary(i => i.TransferNumber, i => i.SystemNo);
                return dic;
            }
            else if (numberType == NumberType.跟踪号)
            {
                var dic = numberInfos.ToDictionary(i => i.TrackingNumber, i => i.SystemNo);
                return dic;
            }
            else return null;
        }

        private async Task CheckBillNameResultExists(string billName)
        {
            bool exists = await _ServiceBillResolveResultRepository.IsExistsAsync(i => i.BillName == billName);
            if (exists)
                throw new Exception($"账单\"{billName}\"的对账结果已存在");
        }

        private void ValidateApplyParam(ApplyResolveServiceBillParam param)
        {
           
            if (param == null)
                throw new Exception("参数不能为空");

            if (string.IsNullOrEmpty(param.FileUrl))
                throw new Exception("账单文件下载地址不能为空");
            if (param.IsBatch == false)
            {
                if (string.IsNullOrWhiteSpace(param.BillName))
                    throw new Exception("解析应用名称不能为空");
            }


            if (string.IsNullOrWhiteSpace(param.FileUrl))
                throw new Exception("解析的文件URL不能为空");
            if (param.SolutionId <= 0)
                throw new Exception("解析方案无效");


        }

        private async Task<ServiceBillResolveParam> CreateResolveParam(int resolveSolutionId, DateTime? billTime, Stream billFile, List<Finance_ServiceBillResolveSolutionRule> rules)
        {
            //var solution = await _solutionRepository.GetAsync(param.SolutionId);
            //if (solution == null)
            //    throw new Exception($"方案不存在:{param.SolutionId}");






            ServiceBillResolveParam resolveParam = new ServiceBillResolveParam
            {
                File = billFile,
                Rules = rules,
                SolutionId = resolveSolutionId,
                BillTime = billTime
            };

            return resolveParam;
        }

        private Finance_ServiceBillResolveApply CreateResolveSolutionApply(int serviceProviderId, string billTypeCode, int resolveSolutionId, string fileUrl, string billName, string batchFileName, string description)
        {
            Finance_ServiceBillResolveApply apply = new Finance_ServiceBillResolveApply
            {
                BillFileUrl = fileUrl,
                CreateTime = DateTime.Now,
                Description = description,
                BillName = billName,
                SolutionId = resolveSolutionId,
                Status = Model.Enums.ResolveStatus.未开始,
                ServiceProviderId = serviceProviderId,
                BillTypeCode = billTypeCode,
                BatchFileName = batchFileName,
            };
            return apply;
        }


        public async Task<List<ServiceBillResolveSolutionDto>> GetAllSolutions(bool? enabled = null)
        {
            Expression<Func<Finance_ServiceBillResolveSolution, bool>> expression = null;
            if (enabled != null)
            {
                expression = i => i.Deleted == false && i.Enabled == enabled;
            }
            else
            {
                expression = i => i.Deleted == false;
            }
            var solutions = await _solutionRepository.QueryAsync(expression);
            var sortedList = solutions.OrderByDescending(i => i.Id);
            var dtos = sortedList.CopyList<ServiceBillResolveSolutionDto>().ToList();
            return dtos;
        }

        public async Task<PageResult<ServiceBillResolveApplyDto>> QueryResoveApplyPage(QueryModel queryModel)
        {
            var solutionEntityPage = await _solutionApplyRepotitory.QueryPagerAsync(queryModel);
            var result = solutionEntityPage.MapperPager<Finance_ServiceBillResolveApply, ServiceBillResolveApplyDto>();

            if (result?.Result?.Any() == true)
            {
                var allSolution = await _solutionRepository.QueryAsync();
                foreach (var r in result.Result)
                {
                    var solution = allSolution?.FirstOrDefault(i => i.Id == r.SolutionId);
                    r.SolutionName = solution?.Name;
                }
            }

            return result;
        }


        public async Task<PageResult<ExpandoObject>> QueryResoveResultPage(QueryResoveResultPageParam param)
        {
            List<string> numbers = null;
            if (!string.IsNullOrWhiteSpace(param.Numbers))
            {
                numbers = param.Numbers.Split(",", StringSplitOptions.RemoveEmptyEntries).ToList();
            }

            PageResult<Finance_ServiceBillResolveResult> entityPage = await _ServiceBillResolveResultRepository.QueryResolveResultPage(param.BillName, numbers, param.PageIndex, param.PageSize);

            var entities = entityPage.Result;

            PageResult<ExpandoObject> resultPage = new PageResult<ExpandoObject> { PageIndex = entityPage.PageIndex, PageSize = entityPage.PageSize, TotalCount = entityPage.TotalCount };

            var numberDataGroups = entities.GroupBy(i => i.Number);

            List<ExpandoObject> dynamicResult = new List<ExpandoObject>();
            foreach (var numberGroup in numberDataGroups)
            {
                var dataNames = numberGroup.GroupBy(i => i.BillDataName).Select(i => i.Key);

                dynamic expando = new ExpandoObject();
                expando.Number = numberGroup.Key;

                var p = expando as IDictionary<String, object>;
                foreach (var name in dataNames)
                {
                    var item = numberGroup.First(i => i.BillDataName == name);
                    var data = item.Data;
                    p[name] = data;
                }
                dynamicResult.Add(expando);
            }
            resultPage.Result = dynamicResult;

            return resultPage;
        }

        public async Task<List<string>> GetBillDataNames(string billName)
        {
            if (string.IsNullOrWhiteSpace(billName))
                throw new Exception("账单名称不能为空");
            List<string> result = await _ServiceBillResolveResultRepository.GetResolveDataNames(billName);
            return result;
        }

        public async Task<IList<ServiceBillResolveSolutionRuleDto>> GetSolutionRules(GetSolutionRulesParam param)
        {
            var rules = await _ruleRepository.QueryAsync(r => r.SolutionId == param.SolutionId);
            var result = rules.CopyList<ServiceBillResolveSolutionRuleDto>();
            return result;
        }

        public async Task<IEnumerable<string>> SearchResolvedBillName(string name)
        {
            var applys = await _solutionApplyRepotitory.QueryAsync(i => i.BillName.Contains(name) && i.Status == ResolveStatus.成功);
            if (applys?.Any() == true)
            {
                var names = applys.OrderByDescending(a => a.CreateTime).Select(a => a.BillName).Distinct();
                return names;
            }
            else return null;
        }

        /// <summary>
        /// 检查方案是否需要解析账单时指定账单时间
        /// </summary>
        /// <param name="param"></param>
        /// <returns></returns>
        public async Task<bool> CheckSetBillTimeOnResolve(CheckSetBillTimeOnResolveParam param)
        {
            if (param == null || param.SolutionId <= 0)
                throw new Exception("检查设置账单时间:参数无效");
            var rules = await _ruleRepository.QueryAsync(r => r.SolutionId == param.SolutionId);
            if (rules?.Any() != true)
                throw new Exception($"检查设置账单时间:该ID的方案无规则，Id:{param.SolutionId}");

            bool rs = rules.Any(r => r.SetBillTimeOnResolve);
            return rs;
        }

        public async Task<bool> RemoveSolutionById(int id)
        {
            var ok1 = await _solutionRepository.UpdateByConditionAsync(i => i.Id == id, new { Deleted = true });
            var ok2 = await _ruleRepository.UpdateByConditionAsync(i => i.SolutionId == id, new { Deleted = true });
            return ok1 && ok2;
        }
    }
}
