﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using NPOI.HSSF.UserModel;
using NPOI.SS.UserModel;
using NPOI.SS.Util;
using NPOI.XSSF.UserModel;
using SMP.Model.Enums;
using SMP.Model.Finance;

namespace SMP.Service.ServiceBillResolve
{

    public class NumberListModeResolver : ServiceBillResoverBase
    {
        public override string ResolverName => "单号明细解析器";




        public override List<ServiceBillResolveResult> ResolveRule(IWorkbook workbook, ISheet sheet, Finance_ServiceBillResolveSolutionRule rule)
        {
            if (workbook == null)
                throw new Exception("工作簿workbook不能为空");
            if (sheet == null)
                throw new Exception("表单Sheet不能为空");

            if (rule.ListingType != BillListingStyle.NumberList)
                throw new Exception($"当前解析规则{rule.ListingType}与解析器{ResolverName}不匹配");

            if (string.IsNullOrWhiteSpace(rule.DataAddressPattern))
                throw new Exception("数值地址模式不能为空");
            if (string.IsNullOrWhiteSpace(rule.NumberAddressPattern))
                throw new Exception("单号地址模式不能为空");

            List<ServiceBillResolveResult> results = new List<ServiceBillResolveResult>();

            AddressPatternInfo numberAp = AnalyzeAddressPattern(sheet, rule.NumberAddressPattern);
            AddressPatternInfo dataAp = AnalyzeAddressPattern(sheet, rule.DataAddressPattern);
            AddressPatternInfo billTimeAp = null;
            if (!rule.SetBillTimeOnResolve && !string.IsNullOrWhiteSpace(rule.BillTimeAddressPattern))
            {
                billTimeAp = AnalyzeAddressPattern(sheet, rule.BillTimeAddressPattern);
            }

            if (numberAp.StartRowNumber <= 0 || dataAp.StartRowNumber <= 0)
            {
                throw new Exception("开始行号必须大于0，并且以1为起始");
            }


            //订单号取固定单元格
            if (numberAp.StartRowNumber == numberAp.StopRowNumber && numberAp.StopSymbol == StopSymbol.CertainRow)
            {
                if (dataAp.StartRowNumber != dataAp.StopRowNumber)
                {
                    throw new Exception("单号为单一单元格时，数据取值不能为单元格范围");
                }

                ServiceBillResolveResult resolveResult = new ServiceBillResolveResult { RuleId = rule.Id, SolutionId = rule.SolutionId, NumberType = rule.NumberType };
                string numberCellAddr = $"{numberAp.ColumnName}{numberAp.StartRowNumber}";
                CellReference numberCellReference = new CellReference(numberCellAddr);
                var numberRow = sheet.GetRow(numberCellReference.Row);
                if (numberRow != null)
                {
                    var cell = numberRow.GetCell(numberCellReference.Col);
                    if (cell != null)
                    {
                        TryGetStringValue(workbook, cell, out string number);
                        if (!string.IsNullOrWhiteSpace(rule.NumberResolveRegx))
                        {
                            var matches = Regex.Matches(number, rule.NumberResolveRegx);
                            if (matches != null && matches.Count > 0)
                            {
                                resolveResult.Number = matches[0].Value;
                            }
                            else
                            {
                                throw new Exception("设置了号码匹配正则，但无匹配结果");
                            }
                        }
                        else
                        {
                            ValidateNumberStrict(number);
                            resolveResult.Number = number;
                        }
                    }
                }

                string dataCellAddr = $"{dataAp.ColumnName}{dataAp.StartRowNumber}";
                CellReference dataCellReference = new CellReference(dataCellAddr);
                var dataRow = sheet.GetRow(dataCellReference.Row);
                if (dataRow != null)
                {
                    var cell = dataRow.GetCell(dataCellReference.Col);
                    if (cell != null)
                    {
                        decimal data = 0;
                        TryGetNumericValue(workbook, cell, rule.DataUseRegx, out data, out string currency);
                        resolveResult.Data = data;
                        resolveResult.Currency = string.IsNullOrWhiteSpace(rule.Currency) ? currency : rule.Currency;
                    }
                    else
                    {
                        throw new Exception($"指定规则无效:单元格不存在:{dataCellAddr}");
                    }
                }
                else
                {
                    throw new Exception($"指定规则无效:行不存在,Row:{dataCellReference.Row}");
                }

                DateTime? billTime = null;
                if (!rule.SetBillTimeOnResolve && billTimeAp != null)
                {
                    string billTimeCellAddr = $"{billTimeAp.ColumnName}{billTimeAp.StartRowNumber}";
                    CellReference billTimeCellReference = new CellReference(billTimeCellAddr);
                    int billTimeCellIdx = billTimeCellReference.Col;
                    var billTimeCell = dataRow.GetCell(billTimeCellIdx);
                    bool billTimeOk = TryGetDateTimeValue(workbook, billTimeCell, out billTime);
                    if (!billTimeOk)
                        throw new Exception($"读取时间失败，时间定为模式:{rule.DataAddressPattern}");
                }
                resolveResult.BillTime = billTime;
                resolveResult.BillDataName = rule.DataName;
                results.Add(resolveResult);
            }
            else
            {
                //多行模式下要求一个单号对应一个数据。
                if (numberAp.StartRowNumber != dataAp.StartRowNumber)
                {
                    throw new Exception($"指定规则无效:多行模式下，单号列和数据列不能对齐");
                }
                if (numberAp.StopRowNumber > 0 && dataAp.StopRowNumber > 0 && numberAp.StopRowNumber != dataAp.StopRowNumber)
                {
                    throw new Exception($"指定规则无效:多行模式下，单号列和数据列的结束行不一致");
                }
                if (numberAp.StopSymbol != dataAp.StopSymbol)
                {
                    throw new Exception($"指定规则无效:多行模式下，单号列和数据列结束行不一致");
                }
                int startRowNum = numberAp.StartRowNumber - 1;
                int stopRowNum = 0;
                if (numberAp.StopSymbol == StopSymbol.CertainRow)
                {
                    stopRowNum = numberAp.StopRowNumber;
                }
                else if (numberAp.StopSymbol == StopSymbol.LastRow)
                {
                    stopRowNum = sheet.LastRowNum;
                }
                else if (numberAp.StopSymbol == StopSymbol.EmptyCell)
                {
                    stopRowNum = int.MaxValue;
                }

                string numberCellAddr = $"{numberAp.ColumnName}0";
                CellReference numberCellReference = new CellReference(numberCellAddr);
                int numberCellIdx = numberCellReference.Col;

                string dataCellAddr = $"{dataAp.ColumnName}0";
                CellReference dataCellReference = new CellReference(dataCellAddr);
                int dataCellIdx = dataCellReference.Col;

                int billTimeCellIdx = 0;
                if (!rule.SetBillTimeOnResolve && billTimeAp != null)
                {
                    string billTimeCellAddr = $"{billTimeAp.ColumnName}0";
                    CellReference billTimeCellReference = new CellReference(billTimeCellAddr);
                    billTimeCellIdx = billTimeCellReference.Col;
                }

                DateTime? fixedBillTime = null;

                for (int n = startRowNum; n <= stopRowNum; n++)
                {
                    var row = sheet.GetRow(n);
                    var numberCell = row?.GetCell(numberCellIdx);
                    if (numberCell == null || string.IsNullOrEmpty(numberCell.ToString()))
                    {
                        if (numberAp.StopSymbol == StopSymbol.CertainRow)
                        {
                            throw new Exception($"在指定的行范围内存在空行:{numberAp.ColumnName}{n}");
                        }
                        else if (numberAp.StopSymbol == StopSymbol.EmptyCell)
                        {
                            break;
                        }
                        else if (numberAp.StopSymbol == StopSymbol.LastRow && n < sheet.LastRowNum)
                        {
                            throw new Exception($"未达到最后一行前出现空行:{numberAp.ColumnName}{n}");
                        }
                    }
                    bool cellMergeChanged = CellMergeChanged(numberCell.IsMergedCell);
                    if (cellMergeChanged)
                    {
                        break;
                    }


                    var dataCell = row.GetCell(dataCellIdx);

                    if (numberCell == null)
                    {
                        throw new Exception($"单号单元格为空,{numberAp.ColumnName}{n}");
                    }
                    if (dataCell == null)
                    {
                        throw new Exception($"数据单元格为空,{dataAp.ColumnName}{n}");
                    }
                    checkCell(numberCell);
                    checkCell(dataCell);

                    TryGetStringValue(workbook, numberCell, out string number);
                    ValidateNumberStrict(number);

                    TryGetNumericValue(workbook, dataCell, rule.DataUseRegx, out decimal data, out string currency);

                    DateTime? billTime = null;
                    if (!rule.SetBillTimeOnResolve && billTimeAp != null)
                    {
                        if (billTimeAp.StopSymbol != StopSymbol.CertainRow)
                        {
                            var billTimeCell = row.GetCell(billTimeCellIdx);
                            bool billTimeOk = TryGetDateTimeValue(workbook, billTimeCell, out billTime);
                            if (!billTimeOk)
                                throw new Exception($"读取时间失败，时间定为模式:{rule.BillTimeAddressPattern},行:{billTimeCell.RowIndex + 1},列:{billTimeCell.ColumnIndex + 1}");
                        }
                        else
                        {
                            if (fixedBillTime == null)
                            {
                                var billTimeCellRef = GetCellReference(billTimeAp.ColumnName, billTimeAp.StartRowNumber);

                                short billTimeCellIndex = GetColumnIndex(billTimeAp);
                                var billTimeRowIndex = billTimeAp.StartRowNumber;
                                var r = sheet.GetRow(billTimeCellRef.Row);
                                if (r == null)
                                    throw new Exception($"读取时间失败,读取行:{billTimeRowIndex}");

                                var billTimeCell = r.GetCell(billTimeCellRef.Col);
                                if (billTimeCell == null)
                                    throw new Exception($"读取时间失败,读取行:{billTimeRowIndex},列:{billTimeCellIndex}");

                                bool billTimeOk = TryGetDateTimeValue(workbook, billTimeCell, out billTime);
                                if (!billTimeOk)
                                    throw new Exception($"读取时间失败，时间定为模式:{rule.BillTimeAddressPattern},行:{billTimeCell.RowIndex + 1},列:{billTimeCell.ColumnIndex + 1}");
                                else
                                    fixedBillTime = billTime;

                            }
                            else
                            {
                                billTime = fixedBillTime.Value;
                            }
                        }
                    }


                    ServiceBillResolveResult resolveResult = new ServiceBillResolveResult
                    {
                        Number = number,
                        Data = data,
                        BillDataName = rule.DataName,
                        NumberType = rule.NumberType,
                        RuleId = rule.Id,
                        SolutionId = rule.SolutionId,
                        Currency = string.IsNullOrWhiteSpace(rule.Currency) ? currency : rule.Currency,
                        BillTime = billTime
                    };
                    results.Add(resolveResult);
                }
            }




            return results;


        }

        private void checkCell(ICell cell)
        {
            if (cell.IsMergedCell)
                throw new Exception($"不支持合并单元格:行:{cell.RowIndex+1},列:{cell.ColumnIndex +1 }");
        }

        private void ValidateNumberStrict(string number)
        {
            bool numberOk = ValidateNumber(number);
            if (!numberOk)
            {
                throw new Exception($"解析到的单号格式不正确:\"{number}\"");
            }
        }

        private bool ValidateNumber(string number)
        {
            if (string.IsNullOrWhiteSpace(number))
                return false;

            return Regex.IsMatch(number, trackingNumberRegex);
        }


    }
}
