﻿using NPOI.SS.UserModel;
using NPOI.XSSF.UserModel;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace SMP.Service.ServiceBillResolve
{
    public interface IServiceBillResolver
    {
        List<ServiceBillResolveResult> Resolve(ServiceBillResolveParam param);
    }

    public class ServiceBillResolver : IServiceBillResolver
    {
        public List<ISheet> GetSheets(IWorkbook workbook, string sheetPattern)
        {
            //shett pattern 可以为:
            //数值: "1"表示第一个sheet
            //名字: "Sheet1" 表示Sheet1这个sheet
            //范围:
            //"1*"表示从第一个一直到最后一个sheet
            //"Sheet1*" 表示从"Sheet1"这个sheet之后的sheet
            //"1-5":表示按索引值第1一直到第5个sheet
            //****SheetA-SheetF 该模式暂不支持****

            if (workbook == null)
                throw new Exception("解析表单失败:工作簿workbook规则不能为空");
            if (string.IsNullOrWhiteSpace(sheetPattern))
                throw new Exception("解析表单失败:表单Sheet规则不能为空");


            SheetRangeInfo sheetRangeInfo = null;

            //"1-5"
            if (Regex.IsMatch(sheetPattern, @"^\d+\-\d+$"))
            {
                var nums = sheetPattern.Split('-');
                int start = int.Parse(nums[0]);
                int stop = int.Parse(nums[1]);
                sheetRangeInfo = new SheetRangeInfo { StartSheetIndex = start, StopSheetIndex = stop };
            }
            else if (Regex.IsMatch(sheetPattern, @"^[\w\W]+\-[\w\W]+$"))
            {
                throw new Exception($"暂不支持该形式的Sheet Pattern:\"{sheetPattern}\"");
            }
            //1* 或 Sheet1*
            else if (Regex.IsMatch(sheetPattern, @"^[\w\W]+\*$"))
            {
                int start = 1;
                int stop = -1;
                var data = sheetPattern.Replace("*", string.Empty);
                if (int.TryParse(data, out start))
                {
                    stop = workbook.NumberOfSheets;
                }
                else
                {
                    var startSheet = workbook.GetSheet(data);
                    if (startSheet != null)
                    {
                        start = workbook.GetSheetIndex(data);
                        start += 1;//获取出来的是npoi的索引，必须加1
                        stop = workbook.NumberOfSheets;
                    }
                    else
                    {
                        throw new Exception($"解析表单失败，不存在Sheet:{data}");
                    }
                }
                sheetRangeInfo = new SheetRangeInfo { StartSheetIndex = start, StopSheetIndex = stop };
            }
            // "1"
            else if (int.TryParse(sheetPattern, out int userStartIdx))
            {
                sheetRangeInfo = new SheetRangeInfo { StartSheetIndex = userStartIdx, StopSheetIndex = userStartIdx };
            }
            else
            {
                int idx = workbook.GetSheetIndex(sheetPattern);
                idx += 1;
                sheetRangeInfo = new SheetRangeInfo { StartSheetIndex = idx, StopSheetIndex = idx };
            }

            if (sheetRangeInfo == null)
                throw new Exception($"解析表单失败，分析规则结果为空");
            if (sheetRangeInfo.StopSheetIndex < sheetRangeInfo.StartSheetIndex)
                throw new Exception($"解析表单失败，模式为");

            if (sheetRangeInfo.StartSheetIndex <= 0 || sheetRangeInfo.StopSheetIndex <= 0)
            {
                throw new Exception($"Sheet序号必须是从1开始");
            }



            List<ISheet> sheets = new List<ISheet>();

            int npoiSheetStart = sheetRangeInfo.StartSheetIndex - 1;
            int npoiSheetStop = sheetRangeInfo.StopSheetIndex - 1;
            for (int i = npoiSheetStart; i <= npoiSheetStop; i++)
            {
                ISheet sheet = workbook.GetSheetAt(i);
                sheets.Add(sheet);
            }
            return sheets;
        }

        public List<ServiceBillResolveResult> Resolve(ServiceBillResolveParam param)
        {
            ValidateResolveParam(param);
            IWorkbook workbook = WorkbookFactory.Create(param.File);
            
            List<ServiceBillResolveResult> resolveResults = new List<ServiceBillResolveResult>();
            foreach (var r in param.Rules)
            {
                var ruleResolver = ServiceBillResolverFactory.GetBillResover(r.ListingType);

                var sheets = GetSheets(workbook, r.SheetPattern);
                if (sheets?.Any() != true)
                {
                    throw new Exception($"找不到表单Sheet,规则:{r.RuleName},Sheet模式:{r.SheetPattern}");
                }
                if (r.SetBillTimeOnResolve == true && param.BillTime == null)
                {
                    throw new Exception($"规则【{r.RuleName}】需要在解析时指定账单时间,但参数中的账单时间为空");
                }

                foreach (var sheet in sheets)
                {
                    var ruleResults = ruleResolver.ResolveRule(workbook, sheet, r);

                    if (ruleResults?.Any() == true)
                    {
                        foreach (var rr in ruleResults)
                        {
                            if (r.SetBillTimeOnResolve == true)
                            {
                                rr.BillTime = param.BillTime;
                            }
                        }
                    }
                    if (ruleResults?.Any() == true)
                    {
                        resolveResults.AddRange(ruleResults);
                    }
                }

            }


            return resolveResults;
        }
        private void ValidateResolveParam(ServiceBillResolveParam param)
        {
            if (param == null)
                throw new Exception("账单解析参数不能为空");
            if (param.File == null)
                throw new Exception("账单文件流不能为空");
            if (param.Rules?.Any() != true)
                throw new Exception("账单解析规则不能为空");
        }
    }
}
