﻿using NPOI.SS;
using NPOI.SS.UserModel;
using NPOI.SS.Util;
using SMP.Model.Enums;
using SMP.Model.Finance;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
namespace SMP.Service.ServiceBillResolve
{
    public abstract class ServiceBillResoverBase : IServiceBillResover
    {
        public const string regAccurate = @"^[a-zA-Z]\d{1,}$";//精确匹配单元格： A1表示A1单元格

        //匹配整列,其中*号代表直到最后一行，+号代表直到空行
        public const string regWholeCol1 = "^[a-zA-Z]+$";//匹配整列,如:A,AZ
        public const string regWholeCol2 = @"^[a-zA-Z]+\*$";//匹配整列，和上一个等价，如:A*
        public const string regWholeCol3 = @"^[a-zA-Z]+\d+\*$";//匹配从第几列开始，直到末尾,当A0*时，与A或者A*等价
        public const string regWholeCol4 = @"^[a-zA-Z]+\+$";//匹配一列，直到空行，如:A+
        public const string regWholeCol5 = @"^[a-zA-Z]+\d+\+$";//匹配一列，从第几行开始直到空行,如:A5+，A0+与A+等价

        public const string regRangeCol = @"^[a-zA-Z]+\d+\-[a-zA-Z]+\d+$";//匹配一个范围，如A5-A10

        public const string charaterMatch = "[a-zA-Z]+";//匹配纯字母
        public const string numericMatch = "[0-9]+";//匹配纯数字
        public const string trackingNumberRegex = @"^[a-zA-Z0-9\s\-]+$";//匹配有效单号
        public const string systemNoRegex = @"^XYYEX[0-9]{10}YQ";
        //public const string amountRegex = @"^[\-0-9.,]*$";//匹配金额，例如 -999.09 或者999,99.09

        public const string MAWBNumberRegex = @"\d{3}\-\d{8}";

        public bool? LastCellMerged { get; set; }

        public virtual AddressPatternInfo AnalyzeAddressPattern(ISheet sheet, string addressPattern)
        {
            var colNameMatchs = Regex.Matches(addressPattern, charaterMatch);
            if (colNameMatchs?.Any() != true)
            {
                throw new Exception($"不能识别地址模式:{addressPattern}");
            }
            string colName = colNameMatchs[0].Value;

            int startRowNum = 1;
            int stopRowNum = 1;
            StopSymbol stop = StopSymbol.LastRow;

            if (Regex.IsMatch(addressPattern, regAccurate))//A1
            {
                string rowIdx = Regex.Matches(addressPattern, numericMatch)[0].Value;
                startRowNum = int.Parse(rowIdx);
                stopRowNum = startRowNum;
                stop = StopSymbol.CertainRow;
            }
            else if (Regex.IsMatch(addressPattern, regWholeCol1))//A
            {
                stop = StopSymbol.LastRow;
            }
            else if (Regex.IsMatch(addressPattern, regWholeCol2))//A*
            {
                stop = StopSymbol.LastRow;
            }
            else if (Regex.IsMatch(addressPattern, regWholeCol3))//A5*
            {
                string startRowStr = Regex.Matches(addressPattern, numericMatch)[0].Value;
                startRowNum = int.Parse(startRowStr);
                stop = StopSymbol.LastRow;
            }
            else if (Regex.IsMatch(addressPattern, regWholeCol4))//A+
            {
                stop = StopSymbol.EmptyCell;
            }
            else if (Regex.IsMatch(addressPattern, regWholeCol5))//A5+
            {
                string startRowStr = Regex.Matches(addressPattern, numericMatch)[0].Value;
                startRowNum = int.Parse(startRowStr);
                stop = StopSymbol.EmptyCell;
            }
            else if (Regex.IsMatch(addressPattern, regRangeCol))//A5-A10
            {
                var segments = regRangeCol.Split("-");

                string startCol = Regex.Matches(segments[0], charaterMatch)[0].Value;
                string endCol = Regex.Matches(segments[1], charaterMatch)[0].Value;
                if (!string.Equals(startCol, endCol, StringComparison.OrdinalIgnoreCase))
                {
                    throw new Exception($"在范围模式下列名必须相同:{addressPattern}");
                }

                string startRowStr = Regex.Matches(segments[0], numericMatch)[0].Value;
                startRowNum = int.Parse(startRowStr);

                string stopRowStr = Regex.Matches(segments[1], numericMatch)[0].Value;
                stopRowNum = int.Parse(stopRowStr);

                stop = StopSymbol.CertainRow;
            }
            //如果是空格或者是最后一行，则停止行为-1,避免出现因根据行号一直循环读取行的情况
            if (stop == StopSymbol.EmptyCell)
            {
                stopRowNum = -1;
            }
            else if (stop == StopSymbol.LastRow)
            {
                stopRowNum = sheet.LastRowNum + 1;
            }
            var result = new AddressPatternInfo
            {
                ColumnName = colName,
                StartRowNumber = startRowNum,
                StopRowNumber = stopRowNum,
                StopSymbol = stop
            };
            return result;
        }

        private bool? lastCellMerged = null;
        public bool CellMergeChanged(bool isCurrentCellMerged)
        {
            if (lastCellMerged == null)
            {
                lastCellMerged = isCurrentCellMerged;
                return false;
            }
            else
            {
                bool changed = lastCellMerged.Value != isCurrentCellMerged;
                lastCellMerged = isCurrentCellMerged;
                return changed;
            }
        }


        public CellReference GetCellReference(string columnName, int row)
        {
            string cellAddr = $"{columnName}{row}";
            CellReference cellReference = new CellReference(cellAddr);
            return cellReference;
        }

        public short GetColumnIndex(AddressPatternInfo addressPattern)
        {
            CellReference cellReference = new CellReference(addressPattern.ColumnName);
            return cellReference.Col;
        }

        public List<IRow> GetRows(ISheet sheet, AddressPatternInfo addressPattern)
        {
            List<IRow> rows = new List<IRow>();
            int stopRowNum = addressPattern.StopRowNumber;
            if (addressPattern.StopSymbol == StopSymbol.LastRow || addressPattern.StopSymbol == StopSymbol.EmptyCell)
                stopRowNum = sheet.LastRowNum + 1;


            for (int r = addressPattern.StartRowNumber; r <= stopRowNum; r++)
            {
                string cellAddr = $"{addressPattern.ColumnName}{r}";
                CellReference cellReference = new CellReference(cellAddr);
                var row = sheet.GetRow(cellReference.Row);
                if (addressPattern.StopSymbol == StopSymbol.EmptyCell)
                {
                    if (row == null)
                        break;
                    else
                    {
                        var cell = row.GetCell(cellReference.Col);
                        if (IsEmptyCell(cell))
                            break;
                    }
                }

                rows.Add(row);
            }
            return rows;
        }

        public bool IsEmptyCell(ICell cell)
        {
            if (cell == null || cell.CellType == CellType.Blank || cell.ToString() == string.Empty)
                return true;
            else return false;
        }

        public string GetFirstStringValue(IWorkbook workbook, ISheet sheet, AddressPatternInfo addressPattern, string valueRegx)
        {
            string result = null;
            for (int r = addressPattern.StartRowNumber; r <= addressPattern.StopRowNumber; r++)
            {
                string numberCellAddr = $"{addressPattern.ColumnName}{r}";
                CellReference numberCellReference = new CellReference(numberCellAddr);

                var row = sheet.GetRow(numberCellReference.Row);
                if (row != null)
                {
                    var cell = row.GetCell(numberCellReference.Col);
                    if (cell != null)
                    {
                        TryGetStringValue(workbook, cell, out string number);
                        if (!string.IsNullOrWhiteSpace(number))
                        {
                            if (!string.IsNullOrWhiteSpace(valueRegx))
                            {
                                var matches = Regex.Matches(number, valueRegx);
                                if (matches?.Any() == true)
                                {
                                    var m = matches.FirstOrDefault(i => !string.IsNullOrWhiteSpace(i.Value));
                                    result = m?.Value;
                                    break;
                                }
                            }
                        }
                    }
                }
            }
            return result;
        }

        public bool TryGetStringValue(IWorkbook workbook, ICell cell, out string value)
        {
            value = null;
            bool ok = false;
            if (cell == null)
            {
                return false;
            }
            else if (cell.CellType == NPOI.SS.UserModel.CellType.String)
            {
                value = cell.StringCellValue;
                ok = true;
            }
            else if (cell.CellType == NPOI.SS.UserModel.CellType.Numeric)
            {
                value = cell.NumericCellValue.ToString();
                ok = true;
            }
            else if (cell.CellType == CellType.Formula)
            {
                IFormulaEvaluator formula = GetFormulaEvaluator(workbook);
                formula.EvaluateInCell(cell);
                ok = TryGetStringValue(workbook, cell, out value);

            }
            return ok;
        }

        public bool TryGetDateTimeValue(IWorkbook workbook, ICell cell, out DateTime? dateTime)
        {
            dateTime = null;
            bool ok = false;
            if (cell == null)
            {
                return false;
            }
            else if (cell.CellType == NPOI.SS.UserModel.CellType.String)
            {
                string cellVal = cell.StringCellValue;

                ok = DateTime.TryParse(cellVal, out DateTime dt);
                if (ok)
                    dateTime = dt;
            }
            else if (cell.CellType == NPOI.SS.UserModel.CellType.Numeric)
            {
                var dt = cell.DateCellValue;
                dateTime = dt;
                ok = true;
            }
            return ok;
        }

        public bool TryGetNumericValue(IWorkbook workbook, ICell cell, bool useRegex, out decimal data, out string currency)
        {
            data = 0;
            currency = string.Empty;
            bool ok = false;
            if (cell == null)
            {
                return false;
            }
            else if (cell.CellType == NPOI.SS.UserModel.CellType.String)
            {
                string strVal = cell.StringCellValue;
                string currencyPattern = "[A-Z]{3}";
                var currMathes = Regex.Matches(strVal, currencyPattern);
                if (currMathes?.Any() == true)
                {
                    currency = currMathes[0].Value;
                    useRegex = true;
                }

                if (useRegex)
                {
                    //有些金额的内容可能带一些货币或说明的文字，所以希望尽量自动地提取出数值内容。
                    var matches = Regex.Matches(strVal, @"(-?[\d\,\.]*)");
                    if (matches.Count > 0)
                    {
                        //这个模式匹配会将空字符也匹配出来，所以要过滤一下
                        var val = matches.FirstOrDefault(m => m.Success && !string.IsNullOrWhiteSpace(m.Value)).Value;
                        ok = decimal.TryParse(val, out data);
                        return ok;
                    }
                    else
                    {
                        return false;
                    }
                }
                else
                {
                    ok = decimal.TryParse(strVal, out data);
                }

            }
            else if (cell.CellType == NPOI.SS.UserModel.CellType.Numeric)
            {
                data = (decimal)cell.NumericCellValue;
                ok = true;
            }
            else if (cell.CellType == CellType.Formula)
            {
                IFormulaEvaluator formula = GetFormulaEvaluator(workbook);
                formula.EvaluateInCell(cell);
                ok = TryGetNumericValue(workbook, cell, useRegex, out data, out currency);
            }
            return ok;
        }

        private IFormulaEvaluator GetFormulaEvaluator(IWorkbook workbook)
        {
            IFormulaEvaluator formula = null;
            if (workbook.SpreadsheetVersion == SpreadsheetVersion.EXCEL97)
            {
                formula = new NPOI.HSSF.UserModel.HSSFFormulaEvaluator(workbook);
            }
            else
            {
                formula = new NPOI.XSSF.UserModel.XSSFFormulaEvaluator(workbook);
            }
            return formula;
        }

        public string GetDefaultNumberRegex(NumberType numberType)
        {
            if (numberType == NumberType.提单号)
            {
                return MAWBNumberRegex;
            }
            else if (numberType == NumberType.跟踪号)
                return trackingNumberRegex;
            else if (numberType == NumberType.系统单号)
                return systemNoRegex;
            else
                throw new Exception($"未设置{numberType}类型单号的正则表达式");
        }

        public abstract List<ServiceBillResolveResult> ResolveRule(IWorkbook workbook, ISheet sheet, Finance_ServiceBillResolveSolutionRule rule);

        public abstract string ResolverName { get; }



    }
}
