﻿using Nest;
using SMP.Data.TimeTask;
using SMP.Model.Dto.DataInterconnection;
using SMP.Model.Enums;
using SMP.Model.Finance;
using SMP.Model.TimeTask;
using SMP.Service.Finance;
using SMP.Service.ThirdPartyRequest;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XYY.Common.Standard;

namespace SMP.Service.TimeTask
{
    #region Iservices
    public interface IFinanceTaskService
    {
        Task<PageResult<Finance_TimeTask>> QueryPage(QueryModel query);
        Task<PageResult<Finance_TimeTypeTaskDetail>> QueryDetailPage(QueryModel query);
        Task<long> Create(Finance_TimeTask model);
        Task<IEnumerable<Finance_TimeTask>> QueryWaitRead();
        Task<bool> ExecuteTaskJob(int taskId, DateTime startTime, DateTime endTime);
        Task RetryExecute(int id, int taskTypeEnum);
        Task<IEnumerable<Finance_TimeTask>> QueryFail();
    }
    #endregion
    #region services
    public class FinanceTaskService : IFinanceTaskService
    {

        private readonly IFinance_TimeTaskRepository _repository;
        private readonly IWMSDataInterconnectionService _wMSDataInterconnection;
        private readonly IOrderCostService _orderCostService;
        public FinanceTaskService(IFinance_TimeTaskRepository repository, IWMSDataInterconnectionService wMSDataInterconnection, IOrderCostService orderCostService)
        {
            _repository = repository;
            this._wMSDataInterconnection = wMSDataInterconnection;
            this._orderCostService = orderCostService;
        }
        public async Task<PageResult<Finance_TimeTask>> QueryPage(QueryModel query)
        {
            var result = await _repository.QueryPager2Async(query);
            return result;
        }
        public async Task<PageResult<Finance_TimeTypeTaskDetail>> QueryDetailPage(QueryModel query)
        {
            var result = await _repository.QueryDetailPage(query);
            return result;
        }
        public async Task RetryExecute(int id, int taskTypeEnum)
        {
            string sql = $" update Finance_TimeTask set TaskStatus={ (int)FinanceEnum.WaitRead},TaskType={taskTypeEnum} where Id={id}";
            await _repository.ExecuteSqlAsync(sql);
        }
        public async Task<long> Create(Finance_TimeTask model)
        {
            return await _repository.InsertAsync(model);
        }
        public async Task<IEnumerable<Finance_TimeTask>> QueryWaitRead()
        {
            var result = await _repository.QueryAsync(x => x.TaskStatus == FinanceEnum.WaitRead);
            var tempResult = result.OrderByDescending(x => x.TaskType).Take(3);
            if (tempResult.Count() > 0)
            {
                if (tempResult.Any(x => string.IsNullOrEmpty(x.LadingBillNumber) && string.IsNullOrEmpty(x.Remark)))
                {
                    //优先执行时间拉取任务，且该类型任务只取一条
                    tempResult = tempResult.Where(x => x.TaskType == TaskTypeEnum.NormalTime).Take(1).ToList();
                }
                //一次只取3条任务
                List<int> Ids = tempResult.Select(x => x.Id).Distinct().ToList();
                await _repository.UpdateTaskStatus(FinanceEnum.WaitExecute, Ids);
            }
            return tempResult;
        }
        public async Task<IEnumerable<Finance_TimeTask>> QueryFail()
        {
            var result = await _repository.QueryAsync(x => x.TaskStatus == FinanceEnum.Fail && x.RetryCount < 4);
            var tempResult = result.OrderBy(x => x.TaskType).Take(3).ToList();//与正常执行数据反向排序
            if (tempResult.Count() > 0)
            {
                if (tempResult.Any(x => string.IsNullOrEmpty(x.LadingBillNumber) && string.IsNullOrEmpty(x.Remark))) { tempResult = tempResult.Take(1).ToList(); }
                foreach (var item in tempResult)
                {
                    //根据任务类型和异常信息，更新执行方式
                    if (string.IsNullOrEmpty(item.LadingBillNumber) && string.IsNullOrEmpty(item.Remark))
                    {
                        item.TaskType = TaskTypeEnum.NormalTime;//时间拉取始终执行全部
                    }
                    else if (item.ResultMessage.Contains("订单获取失败") || item.ResultMessage.Contains("执行异常"))
                    {
                        item.RetryTotal = 0;//重置总次数 整个任务重试，不重置自动执行次数
                    }
                    item.TaskStatus = FinanceEnum.Executing;
                    await _repository.UpdateAsync(item);
                }
            }
            return tempResult;
        }
        public async Task<bool> ExecuteTaskJob(int taskId, DateTime startTime, DateTime endTime)
        {
            Stopwatch watch = new Stopwatch();
            watch.Start();
            var dbModel = await _repository.GetAsync(taskId);
            List<Logistisc_Order> orders = new List<Logistisc_Order>();
            dbModel.TaskStatus = FinanceEnum.Executing;
            try
            {
                switch (dbModel.TaskType)
                {
                    case TaskTypeEnum.NormalTime:
                        orders = await ExecuteByNormalTime(dbModel, startTime, endTime);
                        break;
                    case TaskTypeEnum.DetailScope:
                    case TaskTypeEnum.OrderSeparately:
                        orders = await ExecuteByDetailScope(dbModel);
                        break;
                }
                if (orders != null && orders.Count > 0)
                {
                    //成本预估操作
                    bool result = true;
                    switch (dbModel.TaskType)
                    {
                        case TaskTypeEnum.OrderSeparately:
                            result = await _orderCostService.EestimateOrderSeparately(dbModel, orders);
                            break;
                        default:
                            result = await _orderCostService.EestimateOrderEndCost(dbModel, orders);
                            break;
                    }
                    if (result) { dbModel.ResultMessage = "执行成功！"; dbModel.IsSucceed = true; dbModel.TaskStatus = FinanceEnum.Succeed; }
                    else { dbModel.ResultMessage = "执行过程中有计算异常数据！"; dbModel.IsSucceed = false; dbModel.TaskStatus = FinanceEnum.Fail; }
                }
                else
                {
                    dbModel.ResultMessage = "待执行订单获取失败！"; dbModel.IsSucceed = false; dbModel.TaskStatus = FinanceEnum.Fail;
                }
            }
            catch (Exception ex)
            {
                dbModel.ResultMessage = "执行异常！请整个任务重试！" + ex.Message; dbModel.IsSucceed = false; dbModel.TaskStatus = FinanceEnum.Fail;
            }
            finally
            {
                watch.Stop();
                dbModel.RetryCount++;
                dbModel.RetryTotal+= dbModel.RetryCount;
                dbModel.TimeConsuming = watch.ElapsedMilliseconds;
                await _repository.UpdateAsync(dbModel);
            }
            return dbModel.IsSucceed ?? false;
        }
        #region 执行方式
        private async Task<List<Logistisc_Order>> ExecuteByNormalTime(Finance_TimeTask dbModel, DateTime startTime, DateTime endTime)
        {
            ThirdPartyData_Input input = new ThirdPartyData_Input() { startTime = startTime, endTime = endTime };
            var thirdData = await _wMSDataInterconnection.PullOrderInfo(input);
            if (thirdData == null || thirdData.orders.Count() == 0) { dbModel.ResultMessage = "时间范围内无数据！"; dbModel.IsSucceed = true; dbModel.TaskStatus = FinanceEnum.Succeed; return null; }
            else { dbModel.OrderCount = thirdData.orders.Count(); }
            await _orderCostService.SyncOrderInfo(thirdData.orders);
            return thirdData.orders.ToList();
        }
        private async Task<List<Logistisc_Order>> ExecuteByDetailScope(Finance_TimeTask dbModel)
        {
            var detail = await _repository.GetTaskDetails(dbModel.Id);
            if (detail == null || detail.Count() == 0)
            {
                dbModel.ResultMessage = $"任务记录异常！";
                if (!string.IsNullOrEmpty(dbModel.LadingBillNumber))
                {
                    dbModel.ResultMessage += $"请重新保存提单[{dbModel.LadingBillNumber}]";
                }
                dbModel.IsSucceed = false; dbModel.TaskStatus = FinanceEnum.Fail; return null;
            }
            //已有执行，只取失败数据重跑；提单保存时会更新为全部执行
            if (dbModel.RetryTotal > 0) { detail = detail.Where(x => x.DetailStatus == FinanceEnum.Fail).ToList(); }
            var systemNos = detail.Select(x => x.SystemNo).Distinct().ToList();
            var orders = await _orderCostService.GetOrdersBySystemNos(systemNos);
            return orders.ToList();
        }
        #endregion
    }
    #endregion

}
